"""Additional mixed keys introduced in 2023."""

from ...helps import len_print

BASE_NEW_2023: dict[str, str] = {
    "continental indoor soccer league": "الدوري القاري لكرة القدم داخل الصالات",
    "western indoor soccer league": "الدوري الغربي لكرة القدم داخل الصالات",
    "world indoor soccer league": "الدوري العالمي لكرة القدم داخل الصالات",
    "major indoor soccer league": "الدوري الرئيسي لكرة القدم داخل الصالات",
    "eastern indoor soccer league": "الدوري الشرقي لكرة القدم داخل الصالات",
    "game-of-thrones": "صراع العروش",
    "game of thrones": "صراع العروش",
    "statistics": "إحصائيات",
    "records": "سجلات",
    "records and statistics": "سجلات وإحصائيات",
    "literary characters": "شخصيات أدبية",
    "audiovisual introductions": "استحداثات سمعية بصرية",
    "amusement rides": "ألعاب ملاهي",
    "amusement ride introductions": "استحداثات ألعاب ملاهي",
    "amusement rides introductions": "استحداثات ألعاب ملاهي",
    "amusement parks": "متنزهات ملاهي",
    "amusement park": "متنزهات ملاهي",
    "amusement park introductions": "استحداثات متنزهات ملاهي",
    "amusement parks introductions": "استحداثات متنزهات ملاهي",
    "amusement ride attractions": "ألعاب ملاهي جاذبة",
    "amusement rides attractions": "ألعاب ملاهي جاذبة",
    "amusement park attraction introductions": "استحداثات متنزهات ملاهي جاذبة",
    "amusement park attractions": "متنزهات ملاهي جاذبة",
    "amusement attractions": "ملاهي جاذبة",
    "visitor attractions": "معالم سياحية",
    "tourist attractions": "مواقع جذب سياحي",
    "roadside attractions": "مناطق جذب على جانب الطريق",
    "dubailand": "دبي لاند",
    "freethought": "فكر حر",
    "changyi": "تشانجي",
    "cattle": "الماشية",
    "blindness": "العمى",
    "lgbt rights": "حقوق المثليين",
    "capes": "رؤوس",
    "stalls": "الانهيار",
    "privately held companies": "شركات خاصة",
    "charter-airlines": "طيران عارض",
    "charter airlines": "طيران عارض",
    "defunct charter-airlines": "طيران عارض سابق",
    "psychotherapy": "علاج نفسي",
    "healthcare reform": "إصلاح الرعاية الصحية",
    "pseudonymous writers": "كتاب بأسماء مستعارة",
}


ANTI_KEYS: dict[str, str] = {
    "anti-war": "مناهضة للحرب",
    "anti-revisionist": "مناهضة للتحريفية",
    "anti-communism": "مناهضة للشيوعية",
    "anti-capitalist": "مناهضة للرأسمالية",
    "anti-islam": "مناهضة للإسلام",
    "anti-zionist": "مناهضة للصهيونية",
    "anti-jewish": "مناهضة لليهودية",
    "anti-monopoly": "مناهضة للإحتكار",
    "anti-masonic": "مناهضة للماسونية",
    "anti-smoking": "مناهضة للتدخين",
    "anti-obesity": "مناهضة للسمنة",
    "anti-vaccination": "مناهضة للتطعيم",
    "anti-poverty": "مناهضة للفقر",
    "anti-communists": "مناهضة للشيوعية",
    "anti-racism": "مناهضة للعنصرية",
    "anti-liquor": "مناهضة للخمور",
    "anti-abortion": "مناهضة للإجهاض",
}


ANTI_SUFFIXES: dict[str, str] = {
    "company": "شركات",
    "protests": "احتجاجات",
    "organizations": "منظمات",
    "organization": "منظمات",
    "works": "أعمال",
    "movement": "حركات",
    "albums": "ألبومات",
    "books": "كتب",
    "comic book": "كتب قصص مصورة",
    "comic strips": "شرائط كومكس",
    "comic": "قصص مصورة",
    "comics": "قصص مصورة",
    "marvel comics": "مارفال كومكس",
    "dictionaries": "قواميس",
    "encyclopedias": "موسوعات",
    "films": "أفلام",
    "given names": "أسماء شخصية",
    "graphic novels": "روايات مصورة",
    "inscriptions": "نقوش",
    "literary awards": "جوائز أدبية",
    "magazines": "مجلات",
    "manga": "مانغا",
    "music": "موسيقى",
    "names": "أسماء",
    "newspapers": "صحف",
    "novels": "روايات",
    "novellas": "روايات قصيرة",
    "operas": "أوبيرات",
    "plays": "مسرحيات",
    "poems": "قصائد",
    "publications": "منشورات",
    "radio stations": "محطات إذاعية",
    "short stories": "قصص قصيرة",
    "songs": "أغان",
    "surnames": "ألقاب",
    "video games": "ألعاب فيديو",
    "webcomic": "ويب كومكس",
    "webcomics": "ويب كومكس",
    "websites": "مواقع ويب",
}


AFC_KEYS: dict[str, str] = {
    "afc asian cup qualification": "تصفيات كأس آسيا",
    "fifa world cup qualification (caf)": "تصفيات كأس العالم لكرة القدم (إفريقيا)",
    "fifa world cup qualification (afc)": "تصفيات كأس العالم لكرة القدم (آسيا)",
    "fifa futsal world cup qualification": "تصفيات كأس العالم لكرة الصالات",
    "fifa futsal world cup qualification (afc)": "تصفيات كأس العالم لكرة الصالات (آسيا)",
    "fifa futsal world cup qualification (caf)": "تصفيات كأس العالم لكرة الصالات (إفريقيا)",
    "afc challenge league": "دوري التحدي الآسيوي",
    "association football afc": "كرة القدم في الاتحاد الآسيوي لكرة القدم",
    "association-football afc": "كرة القدم في الاتحاد الآسيوي لكرة القدم",
    "afc elite league": "دوري نخبة ابطال آسيا",
    "afc champions league elite": "دوري نخبة ابطال آسيا",
    "afc champions league two": "دوري أبطال آسيا الثاني",
    "afc cup participants": "مشاركون في كأس الاتحاد الآسيوي لكرة القدم",
    "asian football confederation": "الاتحاد الآسيوي لكرة القدم",
    "afc football": "كرة قدم الاتحاد الآسيوي لكرة القدم",
    "afc cup": "كأس الاتحاد الآسيوي",
    "afc asian cup": "كأس آسيا",
    "afc champions league": "دوري أبطال آسيا",
    "afc asian cup finals": "نهائيات كأس آسيا",
    "afc president's cup": "كأس رئيس الاتحاد الآسيوي",
    "afc solidarity cup": "كأس التضامن الآسيوي",
    "afc women's asian cup": "كأس الأمم الآسيوية لكرة القدم للسيدات",
    "afc u-23 women's asian cup": "كأس آسيا للسيدات تحت 23 سنة",
    "afc u-22 women's asian cup": "كأس آسيا للسيدات تحت 22 سنة",
    "afc u-20 women's asian cup": "كأس آسيا للسيدات تحت 20 سنة",
    "afc u-19 women's asian cup": "كأس آسيا للسيدات تحت 19 سنة",
    "afc u-17 women's asian cup": "كأس آسيا للسيدات تحت 17 سنة",
    "afc u-16 women's asian cup": "كأس آسيا للسيدات تحت 16 سنة",
    "afc women's championship": "بطولة آسيا للسيدات",
    "afc u-22 women's championship": "بطولة آسيا للسيدات تحت 22 سنة",
    "afc u-23 women's championship": "بطولة آسيا للسيدات تحت 23 سنة",
    "afc u-20 women's championship": "بطولة آسيا للسيدات تحت 20 سنة",
    "afc u-19 women's championship": "بطولة آسيا للسيدات تحت 19 سنة",
    "afc u-17 women's championship": "بطولة آسيا للسيدات تحت 17 سنة",
    "afc u-16 women's championship": "بطولة آسيا للسيدات تحت 16 سنة",
    "afc youth championship": "بطولة آسيا للشباب",
    "afc u-23 championship": "بطولة آسيا تحت 23 سنة",
    "afc u-22 championship": "بطولة آسيا للناشئين تحت 22 سنة",
    "afc u-20 championship": "بطولة آسيا للناشئين تحت 20 سنة",
    "afc u-19 championship": "بطولة آسيا للناشئين تحت 19 سنة",
    "afc u-17 championship": "بطولة آسيا للناشئين تحت 17 سنة",
    "afc u-16 championship": "بطولة آسيا للناشئين تحت 16 سنة",
    "afc u-23 asian cup": "كأس آسيا تحت 23 سنة",
    "afc u-22 asian cup": "كأس آسيا للناشئين تحت 22 سنة",
    "afc u-20 asian cup": "كأس آسيا للناشئين تحت 20 سنة",
    "afc u-19 asian cup": "كأس آسيا للناشئين تحت 19 سنة",
    "afc u-17 asian cup": "كأس آسيا للناشئين تحت 17 سنة",
    "afc u-16 asian cup": "كأس آسيا للناشئين تحت 16 سنة",
    "afc futsal championship": "بطولة آسيا لكرة الصالات",
    "afc women's futsal championship": "بطولة آسيا لكرة الصالات للسيدات",
    "afc futsal asian cup": "كأس آسيا لكرة الصالات",
    "afc futsal club championship": "بطولة آسيا لكرة الصالات للأندية",
}


def build_new_2023() -> dict[str, str]:
    """Return the mapping that augments the historical ``NEW_2023`` dictionary."""

    data = dict(BASE_NEW_2023)

    for prefix, prefix_label in ANTI_KEYS.items():  # Category:Anti-war protests in France
        lowered = prefix.lower()
        for suffix, suffix_label in ANTI_SUFFIXES.items():
            data[f"{lowered} {suffix}"] = f"{suffix_label} {prefix_label}"

    data.update({x.lower(): v for x, v in AFC_KEYS.items()})

    return data


NEW_2023: dict[str, str] = build_new_2023()


len_print.data_len(
    "keys_23.py",
    {
        "NEW_2023": NEW_2023,
    },
)
__all__ = ["NEW_2023"]



#
import pytest
from load_one_data import dump_diff, dump_diff_text, dump_same_and_not_same, one_dump_test

from ArWikiCats import resolve_label_ar

test_c_composers_0 = {
    "LGBTQ classical composers": "ملحنون كلاسيكيون مثليون",
    "Italian classical composers of church music": "ملحنو كلاسيكية إيطاليون في موسيقى مسيحية",
    "German classical composers of church music": "ملحنو كلاسيكية ألمان في موسيقى مسيحية",
    "English classical composers of church music": "ملحنو كلاسيكية إنجليز في موسيقى مسيحية",
    "Classical composers of church music": "ملحنون كلاسيكيون من موسيقى مسيحية",
    "American male classical composers": "ملحنون كلاسيكيون أمريكيون",
    "Australian male classical composers": "ملحنون كلاسيكيون أستراليون",
    "Austrian male classical composers": "ملحنون كلاسيكيون نمساويون",
    "Belgian male classical composers": "ملحنون كلاسيكيون بلجيكيون",
    "British male classical composers": "ملحنون كلاسيكيون بريطانيون",
    "Chinese male classical composers": "ملحنون كلاسيكيون صينيون",
    "Czech male classical composers": "ملحنون كلاسيكيون تشيكيون",
    "Danish male classical composers": "ملحنون كلاسيكيون دنماركيون",
    "Dutch male classical composers": "ملحنون كلاسيكيون هولنديون",
    "English male classical composers": "ملحنون كلاسيكيون إنجليز",
    "French male classical composers": "ملحنون كلاسيكيون فرنسيون",
    "German male classical composers": "ملحنون كلاسيكيون ألمان",
    "Irish male classical composers": "ملحنون كلاسيكيون أيرلنديون",
    "Italian male classical composers": "ملحنون كلاسيكيون إيطاليون",
    "Japanese male classical composers": "ملحنون كلاسيكيون يابانيون",
    "Mexican male classical composers": "ملحنون كلاسيكيون مكسيكيون",
    "Polish male classical composers": "ملحنون كلاسيكيون بولنديون",
    "Russian male classical composers": "ملحنون كلاسيكيون روس",
    "Soviet male classical composers": "ملحنون كلاسيكيون سوفيت",
    "Spanish male classical composers": "ملحنون كلاسيكيون إسبان",
    "Mexican classical composers": "ملحنون وملحنات كلاسيكيون مكسيكيون",
    "Polish classical composers": "ملحنون وملحنات كلاسيكيون بولنديون",
    "Russian classical composers": "ملحنون وملحنات كلاسيكيون روس",
    "Soviet classical composers": "ملحنون وملحنات كلاسيكيون سوفيت",
    "Spanish classical composers": "ملحنون وملحنات كلاسيكيون إسبان",
    "German classical composers": "ملحنون وملحنات كلاسيكيون ألمان",
    "Irish classical composers": "ملحنون وملحنات كلاسيكيون أيرلنديون",
    "Italian classical composers": "ملحنون وملحنات كلاسيكيون إيطاليون",
    "Japanese classical composers": "ملحنون وملحنات كلاسيكيون يابانيون",
    "English classical composers": "ملحنون وملحنات كلاسيكيون إنجليز",
    "French classical composers": "ملحنون وملحنات كلاسيكيون فرنسيون",
    "Czech classical composers": "ملحنون وملحنات كلاسيكيون تشيكيون",
    "Dutch classical composers": "ملحنون وملحنات كلاسيكيون هولنديون",
    "Chinese classical composers": "ملحنون وملحنات كلاسيكيون صينيون",
    "Belgian classical composers": "ملحنون وملحنات كلاسيكيون بلجيكيون",
    "British classical composers": "ملحنون وملحنات كلاسيكيون بريطانيون",
    "Australian classical composers": "ملحنون وملحنات كلاسيكيون أستراليون",
    "Austrian classical composers": "ملحنون وملحنات كلاسيكيون نمساويون",
    "American classical composers": "ملحنون وملحنات كلاسيكيون أمريكيون",
}

test_c_composers_3 = {
    "16th-century classical composers": "ملحنون كلاسيكيون في القرن 16",
    "17th-century classical composers": "ملحنون كلاسيكيون في القرن 17",
    "18th-century classical composers from Bohemia": "ملحنون كلاسيكيون من بوهيميا في القرن 18",
    "18th-century classical composers from the Holy Roman Empire": "ملحنون كلاسيكيون من الإمبراطورية الرومانية المقدسة في القرن 18",
    "19th-century classical composers from the Russian Empire": "ملحنون كلاسيكيون من الإمبراطورية الروسية في القرن 19",
    "20th-century classical composers from Northern Ireland": "ملحنون كلاسيكيون من أيرلندا الشمالية في القرن 20",
    "21st-century classical composers from Northern Ireland": "ملحنون كلاسيكيون من أيرلندا الشمالية في القرن 21",
    "African-American male classical composers": "ملحنون كلاسيكيون ذكور أمريكيون أفارقة",
    "African-American women classical composers": "ملحنات كلاسيكيات أمريكيات إفريقيات",
    "American contemporary classical composers": "ملحنون كلاسيكيون معاصرون أمريكيون",
    "American women classical composers": "ملحنات كلاسيكيات أمريكيات",
    "Argentine classical composers": "ملحنون كلاسيكيون أرجنتينيون",
    "Armenian classical composers": "ملحنون كلاسيكيون أرمن",
    "Australian women classical composers": "ملحنات كلاسيكيات أستراليات",
    "Austrian women classical composers": "ملحنات كلاسيكيات نمساويات",
    "Azerbaijani classical composers": "ملحنون كلاسيكيون أذربيجانيون",
    "Brazilian classical composers": "ملحنون كلاسيكيون برازيليون",
    "British contemporary classical composers": "ملحنون كلاسيكيون معاصرون بريطانيون",
    "British women classical composers": "ملحنات كلاسيكيات بريطانيات",
    "Bulgarian classical composers": "ملحنون كلاسيكيون بلغاريون",
    "Canadian classical composers": "ملحنون كلاسيكيون كنديون",
    "Canadian contemporary classical composers": "ملحنون كلاسيكيون معاصرون كنديون",
    "Canadian male classical composers": "ملحنون كلاسيكيون ذكور كنديون",
    "Chilean classical composers": "ملحنون كلاسيكيون تشيليون",
    "Chinese women classical composers": "ملحنات كلاسيكيات صينيات",
    "Classical composers from Catalonia": "ملحنون كلاسيكيون من كتالونيا",
    "Classical composers from Georgia (country)": "ملحنون كلاسيكيون من جورجيا",
    "Classical composers from Northern Ireland": "ملحنون كلاسيكيون من أيرلندا الشمالية",
    "Contemporary classical composers": "ملحنون كلاسيكيون معاصرون",
    "Contemporary classical composers by nationality": "ملحنون كلاسيكيون معاصرون حسب الجنسية",
    "Cuban classical composers": "ملحنون كلاسيكيون كوبيون",
    "Dutch women classical composers": "ملحنات كلاسيكيات هولنديات",
    "English contemporary classical composers": "ملحنون كلاسيكيون معاصرون إنجليز",
    "Estonian classical composers": "ملحنون كلاسيكيون إستونيون",
    "Filipino classical composers": "ملحنون كلاسيكيون فلبينيون",
    "Finnish classical composers": "ملحنون كلاسيكيون فنلنديون",
    "Finnish male classical composers": "ملحنون كلاسيكيون ذكور فنلنديون",
    "Finnish women classical composers": "ملحنات كلاسيكيات فنلنديات",
    "French women classical composers": "ملحنات كلاسيكيات فرنسيات",
    "German contemporary classical composers": "ملحنون كلاسيكيون معاصرون ألمان",
    "German women classical composers": "ملحنات كلاسيكيات ألمانيات",
    "Greek classical composers": "ملحنون كلاسيكيون يونانيون",
    "Guatemalan classical composers": "ملحنون كلاسيكيون غواتيماليون",
    "Hungarian classical composers": "ملحنون كلاسيكيون مجريون",
    "Hungarian male classical composers": "ملحنون كلاسيكيون ذكور مجريون",
    "Indian classical composers": "ملحنون كلاسيكيون هنود",
    "Iranian classical composers": "ملحنون كلاسيكيون إيرانيون",
    "Irish women classical composers": "ملحنات كلاسيكيات أيرلنديات",
    "Israeli classical composers": "ملحنون كلاسيكيون إسرائيليون",
    "Italian women classical composers": "ملحنات كلاسيكيات إيطاليات",
    "Japanese contemporary classical composers": "ملحنون كلاسيكيون معاصرون يابانيون",
    "Japanese women classical composers": "ملحنات كلاسيكيات يابانيات",
    "Jewish American classical composers": "ملحنون كلاسيكيون أمريكيون يهود",
    "Jewish classical composers": "ملحنون كلاسيكيون يهود",
    "Lebanese classical composers": "ملحنون كلاسيكيون لبنانيون",
    "Lithuanian classical composers": "ملحنون كلاسيكيون ليتوانيون",
    "Male classical composers from Northern Ireland": "ملحنون كلاسيكيون ذكور من أيرلندا الشمالية",
    "Maltese classical composers": "ملحنون كلاسيكيون مالطيون",
    "Mexican women classical composers": "ملحنات كلاسيكيات مكسيكيات",
    "New Zealand classical composers": "ملحنون كلاسيكيون نيوزيلنديون",
    "Norwegian classical composers": "ملحنون كلاسيكيون نرويجيون",
    "Norwegian contemporary classical composers": "ملحنون كلاسيكيون معاصرون نرويجيون",
    "Norwegian male classical composers": "ملحنون كلاسيكيون ذكور نرويجيون",
    "Peruvian classical composers": "ملحنون كلاسيكيون بيرويون",
    "Portuguese classical composers": "ملحنون كلاسيكيون برتغاليون",
    "Portuguese male classical composers": "ملحنون كلاسيكيون ذكور برتغاليون",
    "Russian women classical composers": "ملحنات كلاسيكيات روسيات",
    "Scottish classical composers": "ملحنون كلاسيكيون إسكتلنديون",
    "Serbian classical composers": "ملحنون كلاسيكيون صرب",
    "Slovak classical composers": "ملحنون كلاسيكيون سلوفاكيون",
    "South Korean classical composers": "ملحنون كلاسيكيون كوريون جنوبيون",
    "Soviet women classical composers": "ملحنات كلاسيكيات سوفيتيات",
    "Spanish women classical composers": "ملحنات كلاسيكيات إسبانيات",
    "Swedish classical composers": "ملحنون كلاسيكيون سويديون",
    "Swedish male classical composers": "ملحنون كلاسيكيون ذكور سويديون",
    "Swedish women classical composers": "ملحنات كلاسيكيات سويديات",
    "Swiss classical composers": "ملحنون كلاسيكيون سويسريون",
    "Swiss male classical composers": "ملحنون كلاسيكيون ذكور سويسريون",
    "Taiwanese classical composers": "ملحنون كلاسيكيون تايوانيون",
    "Turkish classical composers": "ملحنون كلاسيكيون أتراك",
    "Ukrainian classical composers": "ملحنون كلاسيكيون أوكرانيون",
    "Venezuelan classical composers": "ملحنون كلاسيكيون فنزويليون",
    "Venezuelan women classical composers": "ملحنات كلاسيكيات فنزويليات",
    "Welsh classical composers": "ملحنون كلاسيكيون ويلزيون",
    "Welsh male classical composers": "ملحنون كلاسيكيون ذكور ويلزيون",
    "Women classical composers": "ملحنات كلاسيكيات",
    "Works by British classical composers": "أعمال ملحنون كلاسيكيون بريطانيون",
}

test_c_composers_1 = {
    "17th-century English classical composers": "ملحنون كلاسيكيون إنجليز في القرن 17",
    "17th-century French classical composers": "ملحنون كلاسيكيون فرنسيون في القرن 17",
    "17th-century German classical composers": "ملحنون كلاسيكيون ألمان في القرن 17",
    "17th-century Portuguese classical composers": "ملحنون كلاسيكيون برتغاليون في القرن 17",
    "17th-century Spanish classical composers": "ملحنون كلاسيكيون إسبان في القرن 17",
    "18th-century Austrian classical composers": "ملحنون كلاسيكيون نمساويون في القرن 18",
    "18th-century British classical composers": "ملحنون كلاسيكيون بريطانيون في القرن 18",
    "18th-century classical composers": "ملحنون كلاسيكيون في القرن 18",
    "18th-century French classical composers": "ملحنون كلاسيكيون فرنسيون في القرن 18",
    "18th-century German classical composers": "ملحنون كلاسيكيون ألمان في القرن 18",
    "18th-century Irish classical composers": "ملحنون كلاسيكيون أيرلنديون في القرن 18",
    "18th-century Italian classical composers": "ملحنون كلاسيكيون إيطاليون في القرن 18",
    "18th-century Spanish classical composers": "ملحنون كلاسيكيون إسبان في القرن 18",
    "19th-century American classical composers": "ملحنون كلاسيكيون أمريكيون في القرن 19",
    "19th-century Austrian classical composers": "ملحنون كلاسيكيون نمساويون في القرن 19",
    "19th-century Belgian classical composers": "ملحنون كلاسيكيون بلجيكيون في القرن 19",
    "19th-century British classical composers": "ملحنون كلاسيكيون بريطانيون في القرن 19",
    "19th-century Czech classical composers": "ملحنون كلاسيكيون تشيكيون في القرن 19",
    "19th-century Danish classical composers": "ملحنون كلاسيكيون دنماركيون في القرن 19",
    "19th-century English classical composers": "ملحنون كلاسيكيون إنجليز في القرن 19",
    "19th-century French classical composers": "ملحنون كلاسيكيون فرنسيون في القرن 19",
    "19th-century German classical composers": "ملحنون كلاسيكيون ألمان في القرن 19",
    "19th-century Hungarian classical composers": "ملحنون كلاسيكيون مجريون في القرن 19",
    "19th-century Irish classical composers": "ملحنون كلاسيكيون أيرلنديون في القرن 19",
    "19th-century Italian classical composers": "ملحنون كلاسيكيون إيطاليون في القرن 19",
    "19th-century Norwegian classical composers": "ملحنون كلاسيكيون نرويجيون في القرن 19",
    "19th-century Polish classical composers": "ملحنون كلاسيكيون بولنديون في القرن 19",
    "19th-century Spanish classical composers": "ملحنون كلاسيكيون إسبان في القرن 19",
    "19th-century Swedish classical composers": "ملحنون كلاسيكيون سويديون في القرن 19",
    "20th-century American classical composers": "ملحنون كلاسيكيون أمريكيون في القرن 20",
    "20th-century Argentine classical composers": "ملحنون كلاسيكيون أرجنتينيون في القرن 20",
    "20th-century Armenian classical composers": "ملحنون كلاسيكيون أرمن في القرن 20",
    "20th-century Australian classical composers": "ملحنون كلاسيكيون أستراليون في القرن 20",
    "20th-century Austrian classical composers": "ملحنون كلاسيكيون نمساويون في القرن 20",
    "20th-century Belgian classical composers": "ملحنون كلاسيكيون بلجيكيون في القرن 20",
    "20th-century Brazilian classical composers": "ملحنون كلاسيكيون برازيليون في القرن 20",
    "20th-century British classical composers": "ملحنون كلاسيكيون بريطانيون في القرن 20",
    "20th-century Bulgarian classical composers": "ملحنون كلاسيكيون بلغاريون في القرن 20",
    "20th-century Canadian classical composers": "ملحنون كلاسيكيون كنديون في القرن 20",
    "20th-century Chilean classical composers": "ملحنون كلاسيكيون تشيليون في القرن 20",
    "20th-century Chinese classical composers": "ملحنون كلاسيكيون صينيون في القرن 20",
    "20th-century Cuban classical composers": "ملحنون كلاسيكيون كوبيون في القرن 20",
    "20th-century Czech classical composers": "ملحنون كلاسيكيون تشيكيون في القرن 20",
    "20th-century Danish classical composers": "ملحنون كلاسيكيون دنماركيون في القرن 20",
    "20th-century Dutch classical composers": "ملحنون كلاسيكيون هولنديون في القرن 20",
    "20th-century Egyptian classical composers": "ملحنون كلاسيكيون مصريون في القرن 20",
    "20th-century English classical composers": "ملحنون كلاسيكيون إنجليز في القرن 20",
    "20th-century Estonian classical composers": "ملحنون كلاسيكيون إستونيون في القرن 20",
    "20th-century Finnish classical composers": "ملحنون كلاسيكيون فنلنديون في القرن 20",
    "20th-century French classical composers": "ملحنون كلاسيكيون فرنسيون في القرن 20",
    "20th-century German classical composers": "ملحنون كلاسيكيون ألمان في القرن 20",
    "20th-century Greek classical composers": "ملحنون كلاسيكيون يونانيون في القرن 20",
    "20th-century Hungarian classical composers": "ملحنون كلاسيكيون مجريون في القرن 20",
    "20th-century Irish classical composers": "ملحنون كلاسيكيون أيرلنديون في القرن 20",
    "20th-century Israeli classical composers": "ملحنون كلاسيكيون إسرائيليون في القرن 20",
    "20th-century Italian classical composers": "ملحنون كلاسيكيون إيطاليون في القرن 20",
    "20th-century Japanese classical composers": "ملحنون كلاسيكيون يابانيون في القرن 20",
    "20th-century Lithuanian classical composers": "ملحنون كلاسيكيون ليتوانيون في القرن 20",
    "20th-century Luxembourgian classical composers": "ملحنون كلاسيكيون لوكسمبورغيون في القرن 20",
    "20th-century Mexican classical composers": "ملحنون كلاسيكيون مكسيكيون في القرن 20",
    "20th-century New Zealand classical composers": "ملحنون كلاسيكيون نيوزيلنديون في القرن 20",
    "20th-century Norwegian classical composers": "ملحنون كلاسيكيون نرويجيون في القرن 20",
    "20th-century Polish classical composers": "ملحنون كلاسيكيون بولنديون في القرن 20",
    "20th-century Portuguese classical composers": "ملحنون كلاسيكيون برتغاليون في القرن 20",
    "20th-century Romanian classical composers": "ملحنون كلاسيكيون رومان في القرن 20",
    "20th-century Russian classical composers": "ملحنون كلاسيكيون روس في القرن 20",
    "20th-century Scottish classical composers": "ملحنون كلاسيكيون إسكتلنديون في القرن 20",
    "20th-century South Korean classical composers": "ملحنون كلاسيكيون كوريون جنوبيون في القرن 20",
    "20th-century Spanish classical composers": "ملحنون كلاسيكيون إسبان في القرن 20",
    "20th-century Swedish classical composers": "ملحنون كلاسيكيون سويديون في القرن 20",
    "20th-century Swiss classical composers": "ملحنون كلاسيكيون سويسريون في القرن 20",
    "20th-century Taiwanese classical composers": "ملحنون كلاسيكيون تايوانيون في القرن 20",
    "20th-century Turkish classical composers": "ملحنون كلاسيكيون أتراك في القرن 20",
    "20th-century Ukrainian classical composers": "ملحنون كلاسيكيون أوكرانيون في القرن 20",
    "20th-century Venezuelan classical composers": "ملحنون كلاسيكيون فنزويليون في القرن 20",
    "21st-century American classical composers": "ملحنون كلاسيكيون أمريكيون في القرن 21",
    "21st-century Argentine classical composers": "ملحنون كلاسيكيون أرجنتينيون في القرن 21",
    "21st-century Australian classical composers": "ملحنون كلاسيكيون أستراليون في القرن 21",
    "21st-century Austrian classical composers": "ملحنون كلاسيكيون نمساويون في القرن 21",
    "21st-century Belgian classical composers": "ملحنون كلاسيكيون بلجيكيون في القرن 21",
    "21st-century Brazilian classical composers": "ملحنون كلاسيكيون برازيليون في القرن 21",
    "21st-century British classical composers": "ملحنون كلاسيكيون بريطانيون في القرن 21",
    "21st-century Canadian classical composers": "ملحنون كلاسيكيون كنديون في القرن 21",
    "21st-century Chinese classical composers": "ملحنون كلاسيكيون صينيون في القرن 21",
    "21st-century Czech classical composers": "ملحنون كلاسيكيون تشيكيون في القرن 21",
    "21st-century Danish classical composers": "ملحنون كلاسيكيون دنماركيون في القرن 21",
    "21st-century Dutch classical composers": "ملحنون كلاسيكيون هولنديون في القرن 21",
    "21st-century English classical composers": "ملحنون كلاسيكيون إنجليز في القرن 21",
    "21st-century Estonian classical composers": "ملحنون كلاسيكيون إستونيون في القرن 21",
    "21st-century Finnish classical composers": "ملحنون كلاسيكيون فنلنديون في القرن 21",
    "21st-century French classical composers": "ملحنون كلاسيكيون فرنسيون في القرن 21",
    "21st-century German classical composers": "ملحنون كلاسيكيون ألمان في القرن 21",
    "21st-century Greek classical composers": "ملحنون كلاسيكيون يونانيون في القرن 21",
    "21st-century Indian classical composers": "ملحنون كلاسيكيون هنود في القرن 21",
    "21st-century Irish classical composers": "ملحنون كلاسيكيون أيرلنديون في القرن 21",
    "21st-century Israeli classical composers": "ملحنون كلاسيكيون إسرائيليون في القرن 21",
    "21st-century Italian classical composers": "ملحنون كلاسيكيون إيطاليون في القرن 21",
    "21st-century Japanese classical composers": "ملحنون كلاسيكيون يابانيون في القرن 21",
    "21st-century Mexican classical composers": "ملحنون كلاسيكيون مكسيكيون في القرن 21",
    "21st-century New Zealand classical composers": "ملحنون كلاسيكيون نيوزيلنديون في القرن 21",
    "21st-century Norwegian classical composers": "ملحنون كلاسيكيون نرويجيون في القرن 21",
    "21st-century Polish classical composers": "ملحنون كلاسيكيون بولنديون في القرن 21",
    "21st-century Romanian classical composers": "ملحنون كلاسيكيون رومان في القرن 21",
    "21st-century Russian classical composers": "ملحنون كلاسيكيون روس في القرن 21",
    "21st-century Spanish classical composers": "ملحنون كلاسيكيون إسبان في القرن 21",
    "21st-century Swedish classical composers": "ملحنون كلاسيكيون سويديون في القرن 21",
    "21st-century Swiss classical composers": "ملحنون كلاسيكيون سويسريون في القرن 21",
    "21st-century Venezuelan classical composers": "ملحنون كلاسيكيون فنزويليون في القرن 21",
    "African-American classical composers": "ملحنون كلاسيكيون أمريكيون أفارقة",
    "Argentine women classical composers": "ملحنات كلاسيكيات أرجنتينيات",
    "Basque classical composers": "ملحنون كلاسيكيون باسكيون",
    "Belarusian classical composers": "ملحنون كلاسيكيون بيلاروسيون",
    "Brazilian women classical composers": "ملحنات كلاسيكيات برازيليات",
    "Bulgarian women classical composers": "ملحنات كلاسيكيات بلغاريات",
    "Canadian women classical composers": "ملحنات كلاسيكيات كنديات",
    "Chilean women classical composers": "ملحنات كلاسيكيات تشيليات",
    "Classical composers by century": "ملحنون كلاسيكيون حسب القرن",
    "Classical composers from Bohemia": "ملحنون كلاسيكيون من بوهيميا",
    "Classical composers from the Russian Empire": "ملحنون كلاسيكيون من الإمبراطورية الروسية",
    "Classical composers of African descent": "ملحنون كلاسيكيون من أصل إفريقي",
    "Cuban women classical composers": "ملحنات كلاسيكيات كوبيات",
    "Czech women classical composers": "ملحنات كلاسيكيات تشيكيات",
    "Danish women classical composers": "ملحنات كلاسيكيات دنماركيات",
    "Egyptian classical composers": "ملحنون كلاسيكيون مصريون",
    "English women classical composers": "ملحنات كلاسيكيات إنجليزيات",
    "French contemporary classical composers": "ملحنون كلاسيكيون معاصرون فرنسيون",
    "Icelandic classical composers": "ملحنون كلاسيكيون آيسلنديون",
    "Icelandic women classical composers": "ملحنات كلاسيكيات آيسلنديات",
    "Israeli women classical composers": "ملحنات كلاسيكيات إسرائيليات",
    "Lebanese women classical composers": "ملحنات كلاسيكيات لبنانيات",
    "Luxembourgian classical composers": "ملحنون كلاسيكيون لوكسمبورغيون",
    "Male classical composers": "ملحنون كلاسيكيون ذكور",
    "Moldovan classical composers": "ملحنون كلاسيكيون مولدوفيون",
    "New Zealand women classical composers": "ملحنات كلاسيكيات نيوزيلنديات",
    "Norwegian women classical composers": "ملحنات كلاسيكيات نرويجيات",
    "Polish women classical composers": "ملحنات كلاسيكيات بولنديات",
    "Portuguese women classical composers": "ملحنات كلاسيكيات برتغاليات",
    "Romanian classical composers": "ملحنون كلاسيكيون رومان",
    "Romanian women classical composers": "ملحنات كلاسيكيات رومانيات",
    "Scottish women classical composers": "ملحنات كلاسيكيات إسكتلنديات",
    "Ukrainian women classical composers": "ملحنات كلاسيكيات أوكرانيات",
    "Welsh women classical composers": "ملحنات كلاسيكيات ويلزيات",
    "Women classical composers by nationality": "ملحنات كلاسيكيات حسب الجنسية",
}

test_c_composers_male = {
    "Romanian male classical composers": "ملحنون كلاسيكيون ذكور رومان",
    "Scottish male classical composers": "ملحنون كلاسيكيون ذكور إسكتلنديون",
    "Ukrainian male classical composers": "ملحنون كلاسيكيون ذكور أوكرانيون",
    "Lithuanian male classical composers": "ملحنون كلاسيكيون ذكور ليتوانيون",
    "Male classical composers by nationality": "ملحنون كلاسيكيون ذكور حسب الجنسية",
    "Male classical composers from Bohemia": "ملحنون كلاسيكيون ذكور من بوهيميا",
    "Greek male classical composers": "ملحنون كلاسيكيون ذكور يونانيون",
    "Israeli male classical composers": "ملحنون كلاسيكيون ذكور إسرائيليون",
    "Estonian male classical composers": "ملحنون كلاسيكيون ذكور إستونيون",
    "Egyptian male classical composers": "ملحنون كلاسيكيون ذكور مصريون",
    "Cuban male classical composers": "ملحنون كلاسيكيون ذكور كوبيون",
    "Argentine male classical composers": "ملحنون كلاسيكيون ذكور أرجنتينيون",
    "Armenian male classical composers": "ملحنون كلاسيكيون ذكور أرمن",
    "Brazilian male classical composers": "ملحنون كلاسيكيون ذكور برازيليون",
    "Bulgarian male classical composers": "ملحنون كلاسيكيون ذكور بلغاريون",
    "Chilean male classical composers": "ملحنون كلاسيكيون ذكور تشيليون",
}


to_test = [
    ("test_classical_composers_1", test_c_composers_1),
    ("test_classical_composers_3", test_c_composers_3),
    ("test_classical_composers_4", test_c_composers_male),
]


@pytest.mark.parametrize("category, expected", test_c_composers_1.items(), ids=test_c_composers_1.keys())
@pytest.mark.fast
def test_classical_composers_1(category: str, expected: str) -> None:
    label = resolve_label_ar(category)
    assert label == expected


@pytest.mark.parametrize("name,data", to_test)
@pytest.mark.dump
def test_dump_all(name: str, data: dict[str, str]) -> None:
    expected, diff_result = one_dump_test(data, resolve_label_ar)

    dump_diff(diff_result, name)
    # dump_diff_text (expected, diff_result, name)
    # dump_same_and_not_same(data, diff_result, name, True)
    assert diff_result == expected, f"Differences found: {len(diff_result):,}, len all :{len(data):,}"



"""
Tests
"""

import pytest
from load_one_data import dump_diff, dump_same_and_not_same, one_dump_test

from ArWikiCats.new_resolvers.nationalities_resolvers.nationalities_v2 import resolve_by_nats

nat_match_data = {
    "anti-haitian sentiment": "مشاعر معادية للهايتيون",
    "anti-palestinian sentiment": "مشاعر معادية للفلسطينيون",
    "anti-turkish sentiment": "مشاعر معادية للأتراك",
    "anti-american sentiment": "مشاعر معادية للأمريكيون",
    "anti-czech sentiment": "مشاعر معادية للتشيكيون",
    "anti-japanese sentiment": "مشاعر معادية لليابانيون",
    "anti-asian sentiment": "مشاعر معادية للآسيويون",
    "anti-slovene sentiment": "مشاعر معادية للسلوفينيون",
    "anti-ukrainian sentiment": "مشاعر معادية للأوكرانيون",
    "anti-chechen sentiment": "مشاعر معادية للشيشانيون",
    "anti-mexican sentiment": "مشاعر معادية للمكسيكيون",
    "anti-chinese sentiment": "مشاعر معادية للصينيون",
    "anti-christian sentiment": "مشاعر معادية للمسيحيون",
    "anti-serbian sentiment": "مشاعر معادية للصرب",
    "anti-armenian sentiment": "مشاعر معادية للأرمن",
    "anti-scottish sentiment": "مشاعر معادية للإسكتلنديون",
    "anti-iranian sentiment": "مشاعر معادية للإيرانيون",
    "anti-english sentiment": "مشاعر معادية للإنجليز",
    "anti-hungarian sentiment": "مشاعر معادية للمجريون",
    "anti-greek sentiment": "مشاعر معادية لليونانيون",
}


@pytest.mark.parametrize("category, expected", nat_match_data.items(), ids=nat_match_data.keys())
@pytest.mark.fast
def te_nat_match_data(category: str, expected: str) -> None:
    label = resolve_by_nats(category)
    assert label == expected


ENTERTAINMENT_CASES = [
    ("te_nat_match_data_nats", nat_match_data, resolve_by_nats),
]


@pytest.mark.parametrize("name,data,callback", ENTERTAINMENT_CASES)
@pytest.mark.dump
def test_entertainment(name: str, data: dict[str, str], callback) -> None:
    expected, diff_result = one_dump_test(data, callback)
    # dump_diff(diff_result, name)
    dump_same_and_not_same(data, diff_result, name)
    assert diff_result == expected, f"Differences found: {len(diff_result):,}, len all :{len(data):,}"



import json
from pathlib import Path

from tqdm import tqdm

dir = Path(__file__).parent

f_path = dir / "endings.json"

f_data = {}

json_files = list((dir / "data").glob("*.json"))
json_files.extend(list((dir / "big_data").glob("*.json")))

for file in tqdm(json_files, desc="Processing JSON files"):
    with open(file, "r", encoding="utf-8") as f:
        json_data = json.load(f)

    for key, value in json_data.copy().items():
        if "endings" in key:
            f_data[key] = value
            del json_data[key]
    with open(file, "w", encoding="utf-8") as f:
        json.dump(json_data, f, ensure_ascii=False, indent=4)

with open(f_path, "w", encoding="utf-8") as f:
    json.dump(f_data, f, ensure_ascii=False, indent=4)



# -*- coding: utf-8 -*-
from .ArWikiCats import (
    EventProcessor,
    LoggerWrap,
    batch_resolve_labels,
    config_all_params,
    dump_all_len,
    logger,
    print_memory,
    resolve_arabic_category_label,
)

__all__ = [
    "logger",
    "LoggerWrap",
    "batch_resolve_labels",
    "resolve_arabic_category_label",
    "EventProcessor",
    "do_print_options",
    "print_memory",
    "dump_all_len",
    "config_all_params",
]



"""
Tests
"""

import pytest

from ArWikiCats.legacy_bots.o_bots.parties_bot import get_parties_lab

fast_data = {}


@pytest.mark.parametrize("category, expected", fast_data.items(), ids=fast_data.keys())
@pytest.mark.fast
def test_fast_data(category: str, expected: str) -> None:
    label = get_parties_lab(category)
    assert label == expected


def test_get_parties_lab() -> None:
    # Test with a basic input
    result = get_parties_lab("republican party")
    assert isinstance(result, str)

    result_empty = get_parties_lab("")
    assert isinstance(result_empty, str)

    # Test with various inputs
    result_various = get_parties_lab("some party")
    assert isinstance(result_various, str)



"""
TODO: write tests
"""

import pytest
from load_one_data import dump_diff, one_dump_test

from ArWikiCats.legacy_bots.ma_bots.country_bot import check_historical_prefixes

historical_data = {
    "Defunct national legislatures": "هيئات تشريعية وطنية سابقة",
    "Defunct national football teams": "فرق كرة قدم وطنية سابقة",
    "Defunct National Hockey League teams": "فرق دوري هوكي وطنية سابقة",
    "defunct national basketball league teams": "فرق دوري كرة سلة وطنية سابقة",
}


TEMPORAL_CASES = [
    ("test_yemen_2", historical_data),
]


@pytest.mark.parametrize("category, expected", historical_data.items(), ids=historical_data.keys())
@pytest.mark.fast
def test_historical_data(category: str, expected: str) -> None:
    label = check_historical_prefixes(category)
    assert label == expected


@pytest.mark.parametrize("name,data", TEMPORAL_CASES)
@pytest.mark.dump
def test_all(name: str, data: dict[str, str]) -> None:
    expected, diff_result = one_dump_test(data, check_historical_prefixes)

    dump_diff(diff_result, name)
    assert diff_result == expected, f"Differences found: {len(diff_result):,}, len all :{len(data):,}"



"""
Tests
"""

import pytest

from ArWikiCats.new_resolvers.countries_names_resolvers.countries_names_v2 import resolve_by_countries_names_v2

test_data = {
    "bahamas royal defence force": "قوات الدفاع الملكية البهامية",
    # -------------------------
    # government officials (males)
    # -------------------------
    "united states government officials": "مسؤولون حكوميون أمريكيون",
    "morocco government officials": "مسؤولون حكوميون مغاربة",
    "vanuatu government officials": "مسؤولون حكوميون فانواتيون",
    "vatican government officials": "مسؤولون حكوميون فاتيكانيون",
    "venezuela government officials": "مسؤولون حكوميون فنزويليون",
    "victoria-australia government officials": "مسؤولون حكوميون فيكتوريون",
    "vietnam government officials": "مسؤولون حكوميون فيتناميون",
    "wales government officials": "مسؤولون حكوميون ويلزيون",
    "west germany government officials": "مسؤولون حكوميون ألمانيون غربيون",
    "west india government officials": "مسؤولون حكوميون هنود غربيون",
    "western asia government officials": "مسؤولون حكوميون آسيويون غربيون",
    "western canada government officials": "مسؤولون حكوميون كنديون غربيون",
    "western sahara government officials": "مسؤولون حكوميون صحراويون",
    "yemen government officials": "مسؤولون حكوميون يمنيون",
    "yoruba government officials": "مسؤولون حكوميون يوروبيون",
    "yugoslavia government officials": "مسؤولون حكوميون يوغسلافيون",
    "zaire government officials": "مسؤولون حكوميون زائيريون",
    "zambia government officials": "مسؤولون حكوميون زامبيون",
    "zanzibar government officials": "مسؤولون حكوميون زنجباريون",
    "zimbabwe government officials": "مسؤولون حكوميون زيمبابويون",
    "zulu people government officials": "مسؤولون حكوميون زولو",
    # -------------------------
    # air force (women + article)
    # -------------------------
    "united states air force": "القوات الجوية الأمريكية",
    "new zealand royal air force": "القوات الجوية الملكية النيوزيلندية",
    "china air force": "القوات الجوية الصينية",
    "sri lanka air force": "القوات الجوية السريلانكية",
    "netherlands royal air force": "القوات الجوية الملكية الهولندية",
    "korea air force": "القوات الجوية الكورية",
    "vanuatu air force": "القوات الجوية الفانواتية",
    "vatican air force": "القوات الجوية الفاتيكانية",
    "venezuela air force": "القوات الجوية الفنزويلية",
    "victoria-australia air force": "القوات الجوية الفكتورية",
    "vietnam air force": "القوات الجوية الفيتنامية",
    "wales air force": "القوات الجوية الويلزية",
    "west germany air force": "القوات الجوية الألمانية الغربية",
    "west india air force": "القوات الجوية الهندية الغربية",
    "western asia air force": "القوات الجوية الآسيوية الغربية",
    "western canada air force": "القوات الجوية الكندية الغربية",
    "western sahara air force": "القوات الجوية الصحراوية",
    "yemen air force": "القوات الجوية اليمنية",
    "yoruba air force": "القوات الجوية اليوروبية",
    "yugoslavia air force": "القوات الجوية اليوغسلافية",
    "zaire air force": "القوات الجوية الزائيرية",
    "zambia air force": "القوات الجوية الزامبية",
    "zanzibar air force": "القوات الجوية الزنجبارية",
    "zimbabwe air force": "القوات الجوية الزيمبابوية",
    "zulu people air force": "القوات الجوية الزولية",
    # -------------------------
    # naval force / naval forces / navy / royal navy
    # كلها من قوالب النساء → تحتاج مؤنث + ال
    # -------------------------
    "united states navy": "البحرية الأمريكية",
    "china navy": "البحرية الصينية",
    "pakistan navy": "البحرية الباكستانية",
    "korea navy": "البحرية الكورية",
    "new zealand royal navy": "البحرية الملكية النيوزيلندية",
    "netherlands royal navy": "البحرية الملكية الهولندية",
    "iran navy": "البحرية الإيرانية",
    "sri lanka navy": "البحرية السريلانكية",
    "south yemen navy": "البحرية اليمنية الجنوبية",
    "japan navy": "البحرية اليابانية",
    "united arab emirates navy": "البحرية الإماراتية",
    "benin navy": "البحرية البنينية",
    "cyprus navy": "البحرية القبرصية",
    "brunei royal navy": "البحرية الملكية البرونية",
    "vietnam navy": "البحرية الفيتنامية",
    "yemen navy": "البحرية اليمنية",
    "venezuela navy": "البحرية الفنزويلية",
    "zambia naval force": "البحرية الزامبية",
    "zambia naval forces": "البحرية الزامبية",
    "west germany royal navy": "البحرية الملكية الألمانية الغربية",
    "western sahara navy": "البحرية الصحراوية",
    "yugoslavia naval force": "البحرية اليوغسلافية",
    "zimbabwe naval forces": "البحرية الزيمبابوية",
    # -------------------------
    # civil war (women + إضافة “ال”)
    # -------------------------
    "north yemen civil war": "الحرب الأهلية اليمنية الشمالية",
    "sierra leone civil war": "الحرب الأهلية السيراليونية",
    "myanmar civil war": "الحرب الأهلية الميانمارية",
    "yemen civil war": "الحرب الأهلية اليمنية",
    "vietnam civil war": "الحرب الأهلية الفيتنامية",
    "venezuela civil war": "الحرب الأهلية الفنزويلية",
    "western sahara civil war": "الحرب الأهلية الصحراوية",
    "zimbabwe civil war": "الحرب الأهلية الزيمبابوية",
}


@pytest.mark.parametrize("category, expected", test_data.items(), ids=test_data.keys())
@pytest.mark.fast
def test_get_p17_2(category: str, expected: str) -> None:
    label = resolve_by_countries_names_v2(category)
    assert label == expected



import re

import pytest

from ArWikiCats.translations_formats import FormatData

sample_data_type = tuple[dict[str, str], dict[str, str]]


@pytest.fixture
def sample_data() -> sample_data_type:
    formatted_data = {
        "men's {en} world cup": "كأس العالم للرجال في {ar}",
        "women's {en} championship": "بطولة السيدات في {ar}",
        "{en} records": "سجلات {ar}",
    }

    data_list = {
        "football": "كرة القدم",
        "basketball": "كرة السلة",
        "snooker": "سنوكر",
    }

    return formatted_data, data_list


# --- keys_to_pattern -------------------------------------------------
def test_keys_to_pattern_returns_pattern(sample_data: sample_data_type) -> None:
    formatted_data, data_list = sample_data
    bot = FormatData(formatted_data, data_list, key_placeholder="{en}", value_placeholder="{ar}")
    pattern = bot.keys_to_pattern()
    assert isinstance(pattern, re.Pattern)
    assert pattern.search("football")
    assert pattern.search("snooker")


def test_keys_to_pattern_empty_dict() -> None:
    bot = FormatData({}, {}, key_placeholder="{en}", value_placeholder="{ar}")
    assert bot.keys_to_pattern() is None
    assert bot.pattern is None


# --- match_key -------------------------------------------------------
@pytest.mark.parametrize(
    "category,expected",
    [
        ("men's football world cup", "football"),
        ("women's basketball championship", "basketball"),
        ("unknown sport", ""),
    ],
)
def test_match_key(category: str, expected: str, sample_data: sample_data_type) -> None:
    formatted_data, data_list = sample_data
    bot = FormatData(formatted_data, data_list, key_placeholder="{en}", value_placeholder="{ar}")
    assert bot.match_key(category) == expected


def test_match_key_no_pattern() -> None:
    bot = FormatData({}, {}, key_placeholder="{en}", value_placeholder="{ar}")
    assert bot.match_key("football something") == ""


# --- apply_pattern_replacement ---------------------------------------
@pytest.mark.parametrize(
    "template,sport,expected",
    [
        ("كأس العالم في {ar}", "كرة القدم", "كأس العالم في كرة القدم"),
        ("{ar} بطولة", "كرة السلة", "كرة السلة بطولة"),
        ("بدون متغير", "كرة الطائرة", "بدون متغير"),  # placeholder not found
    ],
    ids=[k for k in range(3)],
)
def test_apply_pattern_replacement(template: str, sport: str, expected: str, sample_data: sample_data_type) -> None:
    bot = FormatData(*sample_data, key_placeholder="{en}", value_placeholder="{ar}")
    assert bot.apply_pattern_replacement(template, sport) == expected


# --- normalize_category ----------------------------------------------
@pytest.mark.parametrize(
    "category,sport_key,expected",
    [
        ("men's football world cup", "football", "men's {en} world cup"),
        ("women's basketball championship", "basketball", "women's {en} championship"),
    ],
)
def test_normalize_category(category: str, sport_key: str, expected: str, sample_data: sample_data_type) -> None:
    bot = FormatData(*sample_data, key_placeholder="{en}", value_placeholder="{ar}")
    assert bot.normalize_category(category, sport_key).lower() == expected.lower()


# --- get_template ----------------------------------------------
def test_get_template_found(sample_data: sample_data_type) -> None:
    formatted_data, data_list = sample_data
    bot = FormatData(formatted_data, data_list, key_placeholder="{en}", value_placeholder="{ar}")
    label = bot.get_template("football", "men's football world cup")
    assert label == "كأس العالم للرجال في {ar}"


def test_get_template_not_found(sample_data: sample_data_type) -> None:
    formatted_data, data_list = sample_data
    bot = FormatData(formatted_data, data_list, key_placeholder="{en}", value_placeholder="{ar}")
    assert bot.get_template("football", "unknown text") == ""


# --- search ----------------------------------------------------------
@pytest.mark.parametrize(
    "category,expected",
    [
        ("men's football world cup", "كأس العالم للرجال في كرة القدم"),
        ("women's basketball championship", "بطولة السيدات في كرة السلة"),
        ("snooker records", "سجلات سنوكر"),
        ("random unrelated", ""),
    ],
    ids=[k for k in range(4)],
)
def test_search_output(category: str, expected: str, sample_data: sample_data_type) -> None:
    bot = FormatData(*sample_data, key_placeholder="{en}", value_placeholder="{ar}")
    result = bot.search(category)
    assert result == expected


def test_search_no_sport_match(sample_data: sample_data_type) -> None:
    bot = FormatData(*sample_data, key_placeholder="{en}", value_placeholder="{ar}")
    assert bot.search("غير موجود") == ""


def test_search_missing_label(sample_data: sample_data_type) -> None:
    formatted_data, data_list = sample_data
    bot = FormatData({}, data_list, key_placeholder="{en}", value_placeholder="{ar}")
    assert bot.search("men's football world cup") == ""


def test_search_missing_sport_label(sample_data: sample_data_type) -> None:
    formatted_data, data_list = sample_data
    del data_list["football"]
    bot = FormatData(formatted_data, data_list, key_placeholder="{en}", value_placeholder="{ar}")
    assert bot.search("men's football world cup") == ""


def test_search_no_pattern() -> None:
    bot = FormatData({}, {}, key_placeholder="{en}", value_placeholder="{ar}")
    assert bot.search("men's football world cup") == ""


# --- Case-insensitivity ----------------------------------------------
def test_case_insensitive_match(sample_data: sample_data_type) -> None:
    bot = FormatData(*sample_data, key_placeholder="{en}", value_placeholder="{ar}")
    result = bot.search("MEN'S FOOTBALL WORLD CUP")
    assert result == "كأس العالم للرجال في كرة القدم"



#
import pytest
from load_one_data import dump_diff, one_dump_test

from ArWikiCats import resolve_arabic_category_label

data_1 = {
    "Category:Non-fiction books about Indigenous peoples of the Americas": "تصنيف:كتب غير خيالية عن شعوب الأمريكتين الأصليين",
    "Category:Zimbabwean non-fiction writers": "تصنيف:كتاب غير روائيين زيمبابويون",
    "Category:10th-century non-fiction writers": "تصنيف:كتاب غير روائيين في القرن 10",
    "Category:11th-century Indian non-fiction writers": "تصنيف:كتاب غير روائيين هنود في القرن 11",
    "Category:11th-century non-fiction writers": "تصنيف:كتاب غير روائيين في القرن 11",
    "Category:12th-century Indian non-fiction writers": "تصنيف:كتاب غير روائيين هنود في القرن 12",
    "Category:12th-century non-fiction writers": "تصنيف:كتاب غير روائيين في القرن 12",
    "Category:13th-century Indian non-fiction writers": "تصنيف:كتاب غير روائيين هنود في القرن 13",
    "Category:13th-century non-fiction writers": "تصنيف:كتاب غير روائيين في القرن 13",
    "Category:14th-century Indian non-fiction writers": "تصنيف:كتاب غير روائيين هنود في القرن 14",
    "Category:14th-century non-fiction writers": "تصنيف:كتاب غير روائيين في القرن 14",
    "Category:15th-century Indian non-fiction writers": "تصنيف:كتاب غير روائيين هنود في القرن 15",
    "Category:15th-century non-fiction writers": "تصنيف:كتاب غير روائيين في القرن 15",
    "Category:16th-century Danish non-fiction writers": "تصنيف:كتاب غير روائيين دنماركيون في القرن 16",
    "Category:16th-century English non-fiction writers": "تصنيف:كتاب غير روائيين إنجليز في القرن 16",
    "Category:16th-century Indian non-fiction writers": "تصنيف:كتاب غير روائيين هنود في القرن 16",
    "Category:16th-century non-fiction writers": "تصنيف:كتاب غير روائيين في القرن 16",
    "Category:17th-century American non-fiction writers": "تصنيف:كتاب غير روائيين أمريكيون في القرن 17",
    "Category:17th-century Danish non-fiction writers": "تصنيف:كتاب غير روائيين دنماركيون في القرن 17",
    "Category:17th-century English non-fiction writers": "تصنيف:كتاب غير روائيين إنجليز في القرن 17",
    "Category:17th-century French non-fiction writers": "تصنيف:كتاب غير روائيين فرنسيون في القرن 17",
    "Category:17th-century Indian non-fiction writers": "تصنيف:كتاب غير روائيين هنود في القرن 17",
    "Category:17th-century non-fiction writers": "تصنيف:كتاب غير روائيين في القرن 17",
    "Category:18th-century American non-fiction writers": "تصنيف:كتاب غير روائيين أمريكيون في القرن 18",
    "Category:18th-century British non-fiction writers": "تصنيف:كتاب غير روائيين بريطانيون في القرن 18",
    "Category:18th-century Canadian non-fiction writers": "تصنيف:كتاب غير روائيين كنديون في القرن 18",
    "Category:18th-century English non-fiction writers": "تصنيف:كتاب غير روائيين إنجليز في القرن 18",
    "Category:18th-century Indian non-fiction writers": "تصنيف:كتاب غير روائيين هنود في القرن 18",
    "Category:18th-century non-fiction writers": "تصنيف:كتاب غير روائيين في القرن 18",
    "Category:19th-century American non-fiction writers": "تصنيف:كتاب غير روائيين أمريكيون في القرن 19",
    "Category:19th-century British non-fiction writers": "تصنيف:كتاب غير روائيين بريطانيون في القرن 19",
    "Category:19th-century Canadian non-fiction writers": "تصنيف:كتاب غير روائيين كنديون في القرن 19",
    "Category:19th-century English non-fiction writers": "تصنيف:كتاب غير روائيين إنجليز في القرن 19",
    "Category:19th-century French non-fiction writers": "تصنيف:كتاب غير روائيين فرنسيون في القرن 19",
    "Category:19th-century German non-fiction writers": "تصنيف:كتاب غير روائيين ألمان في القرن 19",
    "Category:19th-century Indian non-fiction writers": "تصنيف:كتاب غير روائيين هنود في القرن 19",
    "Category:19th-century Irish non-fiction writers": "تصنيف:كتاب غير روائيين أيرلنديون في القرن 19",
    "Category:19th-century non-fiction writers": "تصنيف:كتاب غير روائيين في القرن 19",
    "Category:19th-century non-fiction writers by nationality": "تصنيف:كتاب غير روائيين حسب الجنسية في القرن 19",
    "Category:19th-century non-fiction writers from the Russian Empire": "تصنيف:كتاب غير روائيين روس في القرن 19",
    "Category:20th-century American non-fiction writers": "تصنيف:كتاب غير روائيين أمريكيون في القرن 20",
    "Category:20th-century Australian non-fiction writers": "تصنيف:كتاب غير روائيين أستراليون في القرن 20",
    "Category:20th-century Belgian non-fiction writers": "تصنيف:كتاب غير روائيين بلجيكيون في القرن 20",
    "Category:20th-century British non-fiction writers": "تصنيف:كتاب غير روائيين بريطانيون في القرن 20",
    "Category:20th-century Canadian non-fiction writers": "تصنيف:كتاب غير روائيين كنديون في القرن 20",
    "Category:20th-century Chilean non-fiction writers": "تصنيف:كتاب غير روائيين تشيليون في القرن 20",
    "Category:20th-century Croatian non-fiction writers": "تصنيف:كتاب غير روائيين كروات في القرن 20",
    "Category:20th-century Danish non-fiction writers": "تصنيف:كتاب غير روائيين دنماركيون في القرن 20",
    "Category:20th-century Dutch non-fiction writers": "تصنيف:كتاب غير روائيين هولنديون في القرن 20",
    "Category:20th-century English non-fiction writers": "تصنيف:كتاب غير روائيين إنجليز في القرن 20",
    "Category:20th-century French non-fiction writers": "تصنيف:كتاب غير روائيين فرنسيون في القرن 20",
    "Category:20th-century German non-fiction writers": "تصنيف:كتاب غير روائيين ألمان في القرن 20",
    "Category:20th-century Indian non-fiction writers": "تصنيف:كتاب غير روائيين هنود في القرن 20",
    "Category:20th-century Irish non-fiction writers": "تصنيف:كتاب غير روائيين أيرلنديون في القرن 20",
    "Category:20th-century Israeli non-fiction writers": "تصنيف:كتاب غير روائيين إسرائيليون في القرن 20",
    "Category:20th-century Italian non-fiction writers": "تصنيف:كتاب غير روائيين إيطاليون في القرن 20",
    "Category:20th-century Lithuanian non-fiction writers": "تصنيف:كتاب غير روائيين ليتوانيون في القرن 20",
    "Category:20th-century Norwegian non-fiction writers": "تصنيف:كتاب غير روائيين نرويجيون في القرن 20",
    "Category:20th-century Polish non-fiction writers": "تصنيف:كتاب غير روائيين بولنديون في القرن 20",
    "Category:20th-century Russian non-fiction writers": "تصنيف:كتاب غير روائيين روس في القرن 20",
    "Category:20th-century Swedish non-fiction writers": "تصنيف:كتاب غير روائيين سويديون في القرن 20",
    "Category:20th-century Swiss non-fiction writers": "تصنيف:كتاب غير روائيين سويسريون في القرن 20",
    "Category:20th-century non-fiction writers": "تصنيف:كتاب غير روائيين في القرن 20",
    "Category:20th-century non-fiction writers by nationality": "تصنيف:كتاب غير روائيين حسب الجنسية في القرن 20",
    "Category:21st-century American non-fiction writers": "تصنيف:كتاب غير روائيين أمريكيون في القرن 21",
    "Category:21st-century Australian non-fiction writers": "تصنيف:كتاب غير روائيين أستراليون في القرن 21",
    "Category:21st-century Belgian non-fiction writers": "تصنيف:كتاب غير روائيين بلجيكيون في القرن 21",
    "Category:21st-century British non-fiction writers": "تصنيف:كتاب غير روائيين بريطانيون في القرن 21",
    "Category:21st-century Canadian non-fiction writers": "تصنيف:كتاب غير روائيين كنديون في القرن 21",
    "Category:21st-century Chilean non-fiction writers": "تصنيف:كتاب غير روائيين تشيليون في القرن 21",
    "Category:21st-century Croatian non-fiction writers": "تصنيف:كتاب غير روائيين كروات في القرن 21",
    "Category:21st-century Danish non-fiction writers": "تصنيف:كتاب غير روائيين دنماركيون في القرن 21",
    "Category:21st-century Dutch non-fiction writers": "تصنيف:كتاب غير روائيين هولنديون في القرن 21",
    "Category:21st-century English non-fiction writers": "تصنيف:كتاب غير روائيين إنجليز في القرن 21",
    "Category:21st-century Finnish non-fiction writers": "تصنيف:كتاب غير روائيين فنلنديون في القرن 21",
    "Category:21st-century French non-fiction writers": "تصنيف:كتاب غير روائيين فرنسيون في القرن 21",
    "Category:21st-century German non-fiction writers": "تصنيف:كتاب غير روائيين ألمان في القرن 21",
    "Category:21st-century Indian non-fiction writers": "تصنيف:كتاب غير روائيين هنود في القرن 21",
    "Category:21st-century Irish non-fiction writers": "تصنيف:كتاب غير روائيين أيرلنديون في القرن 21",
    "Category:21st-century Israeli non-fiction writers": "تصنيف:كتاب غير روائيين إسرائيليون في القرن 21",
    "Category:21st-century Italian non-fiction writers": "تصنيف:كتاب غير روائيين إيطاليون في القرن 21",
    "Category:21st-century Polish non-fiction writers": "تصنيف:كتاب غير روائيين بولنديون في القرن 21",
    "Category:21st-century Russian non-fiction writers": "تصنيف:كتاب غير روائيين روس في القرن 21",
    "Category:21st-century Scottish non-fiction writers": "تصنيف:كتاب غير روائيين إسكتلنديون في القرن 21",
    "Category:21st-century Spanish non-fiction writers": "تصنيف:كتاب غير روائيين إسبان في القرن 21",
    "Category:21st-century Swiss non-fiction writers": "تصنيف:كتاب غير روائيين سويسريون في القرن 21",
    "Category:21st-century non-fiction writers": "تصنيف:كتاب غير روائيين في القرن 21",
    "Category:21st-century non-fiction writers by nationality": "تصنيف:كتاب غير روائيين حسب الجنسية في القرن 21",
    "Category:21st-century non-fiction writers from Northern Ireland": "تصنيف:كتاب غير روائيين من أيرلندا الشمالية في القرن 21",
    "Category:2nd-century Indian non-fiction writers": "تصنيف:كتاب غير روائيين هنود في القرن 2",
    "Category:2nd-century non-fiction writers": "تصنيف:كتاب غير روائيين في القرن 2",
    "Category:3rd-century Indian non-fiction writers": "تصنيف:كتاب غير روائيين هنود في القرن 3",
    "Category:3rd-century non-fiction writers": "تصنيف:كتاب غير روائيين في القرن 3",
    "Category:4th-century Indian non-fiction writers": "تصنيف:كتاب غير روائيين هنود في القرن 4",
    "Category:4th-century non-fiction writers": "تصنيف:كتاب غير روائيين في القرن 4",
    "Category:5th-century Indian non-fiction writers": "تصنيف:كتاب غير روائيين هنود في القرن 5",
    "Category:5th-century non-fiction writers": "تصنيف:كتاب غير روائيين في القرن 5",
    "Category:6th-century Indian non-fiction writers": "تصنيف:كتاب غير روائيين هنود في القرن 6",
    "Category:6th-century non-fiction writers": "تصنيف:كتاب غير روائيين في القرن 6",
    "Category:7th-century Indian non-fiction writers": "تصنيف:كتاب غير روائيين هنود في القرن 7",
    "Category:7th-century non-fiction writers": "تصنيف:كتاب غير روائيين في القرن 7",
    "Category:8th-century Indian non-fiction writers": "تصنيف:كتاب غير روائيين هنود في القرن 8",
    "Category:8th-century non-fiction writers": "تصنيف:كتاب غير روائيين في القرن 8",
    "Category:9th-century Indian non-fiction writers": "تصنيف:كتاب غير روائيين هنود في القرن 9",
    "Category:9th-century non-fiction writers": "تصنيف:كتاب غير روائيين في القرن 9",
    "Category:African-American non-fiction writers": "تصنيف:كتاب غير روائيين أفارقة أمريكيون",
    "Category:Albanian non-fiction writers": "تصنيف:كتاب غير روائيين ألبان",
    "Category:American male non-fiction writers": "تصنيف:كتاب غير روائيين ذكور أمريكيون",
    "Category:American non-fiction crime writers": "تصنيف:كتاب جريمة غير روائيين أمريكيون",
    "Category:American non-fiction environmental writers": "تصنيف:كتاب بيئة غير روائيين أمريكيون",
    "Category:American non-fiction writers": "تصنيف:كتاب غير روائيين أمريكيون",
    "Category:American non-fiction writers by century": "تصنيف:كتاب غير روائيين أمريكيون حسب القرن",
    "Category:Angolan non-fiction writers": "تصنيف:كتاب غير روائيين أنغوليون",
    "Category:Arab non-fiction writers": "تصنيف:كتاب غير روائيين عرب",
    "Category:Argentine non-fiction writers": "تصنيف:كتاب غير روائيين أرجنتينيون",
    "Category:Armenian non-fiction writers": "تصنيف:كتاب غير روائيين أرمن",
    "Category:Australian male non-fiction writers": "تصنيف:كتاب غير روائيين ذكور أستراليون",
    "Category:Australian non-fiction crime writers": "تصنيف:كتاب جريمة غير روائيين أستراليون",
    "Category:Australian non-fiction writers": "تصنيف:كتاب غير روائيين أستراليون",
    "Category:Austrian non-fiction writers": "تصنيف:كتاب غير روائيين نمساويون",
    "Category:Azerbaijani non-fiction writers": "تصنيف:كتاب غير روائيين أذربيجانيون",
    "Category:Bangladeshi non-fiction writers": "تصنيف:كتاب غير روائيين بنغلاديشيون",
    "Category:Belarusian non-fiction writers": "تصنيف:كتاب غير روائيين بيلاروسيون",
    "Category:Belgian male non-fiction writers": "تصنيف:كتاب غير روائيين ذكور بلجيكيون",
    "Category:Belgian non-fiction writers": "تصنيف:كتاب غير روائيين بلجيكيون",
    "Category:Belgian non-fiction writers by century": "تصنيف:كتاب غير روائيين بلجيكيون حسب القرن",
    "Category:Beninese non-fiction writers": "تصنيف:كتاب غير روائيين بنينيون",
    "Category:Bermudian non-fiction writers": "تصنيف:كتاب غير روائيين برموديون",
    "Category:Bolivian non-fiction writers": "تصنيف:كتاب غير روائيين بوليفيون",
    "Category:Bosnia and Herzegovina non-fiction writers": "تصنيف:كتاب غير روائيين بوسنيون",
    "Category:Botswana non-fiction writers": "تصنيف:كتاب غير روائيين بوتسوانيون",
    "Category:Brazilian non-fiction writers": "تصنيف:كتاب غير روائيين برازيليون",
    "Category:British male non-fiction writers": "تصنيف:كتاب غير روائيين ذكور بريطانيون",
    "Category:British non-fiction crime writers": "تصنيف:كتاب جريمة غير روائيين بريطانيون",
    "Category:British non-fiction writers": "تصنيف:كتاب غير روائيين بريطانيون",
    "Category:British non-fiction writers by century": "تصنيف:كتاب غير روائيين بريطانيون حسب القرن",
    "Category:Burkinabe non-fiction writers": "تصنيف:كتاب غير روائيين بوركينابيون",
    "Category:Burkinabé non-fiction writers": "تصنيف:كتاب غير روائيين بوركينابيون",
    "Category:Burmese non-fiction writers": "تصنيف:كتاب غير روائيين بورميون",
    "Category:Cameroonian non-fiction writers": "تصنيف:كتاب كاميرونيون غير روائيين",
    "Category:Canadian male non-fiction writers": "تصنيف:كتاب غير روائيين ذكور كنديون",
    "Category:Canadian non-fiction writers": "تصنيف:كتاب غير روائيين كنديون",
    "Category:Canadian non-fiction writers by century": "تصنيف:كتاب غير روائيين كنديون حسب القرن",
    "Category:Canadian non-fiction writers in French": "تصنيف:كتاب غير روائيين كنديون في الفرنسية",
    "Category:Central African Republic non-fiction writers": "تصنيف:كتاب غير روائيين أفارقة أوسطيون",
    "Category:Chadian non-fiction writers": "تصنيف:كتاب غير روائيين تشاديون",
    "Category:Chilean non-fiction writers": "تصنيف:كتاب غير روائيين تشيليون",
    "Category:Chinese non-fiction writers": "تصنيف:كتاب غير روائيين صينيون",
    "Category:Colombian non-fiction writers": "تصنيف:كتاب غير روائيين كولومبيون",
    "Category:Croatian non-fiction writers by century": "تصنيف:كتاب غير روائيين كروات حسب القرن",
    "Category:Cuban non-fiction writers": "تصنيف:كتاب غير روائيين كوبيون",
    "Category:Cypriot non-fiction writers": "تصنيف:كتاب غير روائيين قبرصيون",
    "Category:Czech non-fiction writers": "تصنيف:كتاب غير روائيين تشيكيون",
    "Category:Danish male non-fiction writers": "تصنيف:كتاب غير روائيين ذكور دنماركيون",
    "Category:Danish non-fiction writers": "تصنيف:كتاب غير روائيين دنماركيون",
    "Category:Danish non-fiction writers by century": "تصنيف:كتاب غير روائيين دنماركيون حسب القرن",
    "Category:Democratic republic of congo non-fiction writers": "تصنيف:كتاب غير روائيين كونغويون ديمقراطيون",
    "Category:Dutch non-fiction writers": "تصنيف:كتاب غير روائيين هولنديون",
    "Category:Dutch non-fiction writers by century": "تصنيف:كتاب غير روائيين هولنديون حسب القرن",
    "Category:Egyptian non-fiction writers": "تصنيف:كتاب غير روائيين مصريون",
    "Category:Emirati non-fiction writers": "تصنيف:كتاب غير روائيين إماراتيون",
    "Category:English male non-fiction writers": "تصنيف:كتاب غير روائيين ذكور إنجليز",
    "Category:English non-fiction crime writers": "تصنيف:كتاب جريمة غير روائيين إنجليز",
    "Category:English non-fiction writers by century": "تصنيف:كتاب غير روائيين إنجليز حسب القرن",
    "Category:Eritrean non-fiction writers": "تصنيف:كتاب غير روائيين إريتريون",
    "Category:Estonian non-fiction writers": "تصنيف:كتاب غير روائيين إستونيون",
    "Category:Ethiopian non-fiction writers": "تصنيف:كتاب غير روائيين إثيوبيون",
    "Category:Filipino non-fiction writers": "تصنيف:كتاب غير روائيين فلبينيون",
    "Category:Finnish non-fiction writers": "تصنيف:كتاب غير روائيين فنلنديون",
    "Category:Finnish non-fiction writers by century": "تصنيف:كتاب غير روائيين فنلنديون حسب القرن",
    "Category:French male non-fiction writers": "تصنيف:كتاب غير روائيين ذكور فرنسيون",
    "Category:French non-fiction writers": "تصنيف:كتاب غير روائيين فرنسيون",
    "Category:French non-fiction writers by century": "تصنيف:كتاب غير روائيين فرنسيون حسب القرن",
    "Category:Gambian non-fiction writers": "تصنيف:كتاب غير روائيين غامبيون",
    "Category:German male non-fiction writers": "تصنيف:كتاب غير روائيين ذكور ألمان",
    "Category:German non-fiction writers": "تصنيف:كتاب غير روائيين ألمان",
    "Category:German non-fiction writers by century": "تصنيف:كتاب غير روائيين ألمان حسب القرن",
    "Category:Ghanaian non-fiction writers": "تصنيف:كتاب غير روائيين غانيون",
    "Category:Greek non-fiction writers": "تصنيف:كتاب غير روائيين يونانيون",
    "Category:Guadeloupean non-fiction writers": "تصنيف:كتاب غير روائيين غواديلوبيون",
    "Category:Guyanese non-fiction writers": "تصنيف:كتاب غير روائيين غيانيون",
    "Category:Haitian non-fiction writers": "تصنيف:كتاب غير روائيين هايتيون",
    "Category:Hungarian non-fiction writers": "تصنيف:كتاب غير روائيين مجريون",
    "Category:Icelandic non-fiction writers": "تصنيف:كتاب غير روائيين آيسلنديون",
    "Category:Indian male non-fiction writers": "تصنيف:كتاب غير روائيين ذكور هنود",
    "Category:Indian non-fiction environmental writers": "تصنيف:كتاب بيئة غير روائيين هنود",
    "Category:Indian non-fiction writers": "تصنيف:كتاب غير روائيين هنود",
    "Category:Indian non-fiction writers by century": "تصنيف:كتاب غير روائيين هنود حسب القرن",
    "Category:Indonesian non-fiction writers": "تصنيف:كتاب غير روائيين إندونيسيون",
    "Category:Iranian non-fiction writers": "تصنيف:كتاب غير روائيين إيرانيون",
    "Category:Iraqi non-fiction writers": "تصنيف:كتاب غير روائيين عراقيون",
    "Category:Irish male non-fiction writers": "تصنيف:كتاب غير روائيين ذكور أيرلنديون",
    "Category:Irish non-fiction writers": "تصنيف:كتاب غير روائيين أيرلنديون",
    "Category:Irish non-fiction writers by century": "تصنيف:كتاب غير روائيين أيرلنديون حسب القرن",
    "Category:Israeli non-fiction writers": "تصنيف:كتاب غير روائيين إسرائيليون",
    "Category:Israeli non-fiction writers by century": "تصنيف:كتاب غير روائيين إسرائيليون حسب القرن",
    "Category:Italian male non-fiction writers": "تصنيف:كتاب غير روائيين ذكور إيطاليون",
    "Category:Italian non-fiction writers": "تصنيف:كتاب غير روائيين إيطاليون",
    "Category:Italian non-fiction writers by century": "تصنيف:كتاب غير روائيين إيطاليون حسب القرن",
    "Category:Jamaican non-fiction writers": "تصنيف:كتاب غير روائيين جامايكيون",
    "Category:Japanese non-fiction writers": "تصنيف:كتاب غير روائيين يابانيون",
    "Category:Japanese non-fiction writers by century": "تصنيف:كتاب غير روائيين يابانيون حسب القرن",
    "Category:Jewish American non-fiction writers": "تصنيف:كتاب غير روائيين أمريكيون يهود",
    "Category:Jewish non-fiction writers": "تصنيف:كتاب غير روائيين يهود",
    "Category:Kenyan non-fiction writers": "تصنيف:كتاب غير روائيين كينيون",
    "Category:Korean non-fiction writers": "تصنيف:كتاب غير روائيين كوريون",
    "Category:Kosovan non-fiction writers": "تصنيف:كتاب غير روائيين كوسوفيون",
    "Category:Kuwaiti non-fiction writers": "تصنيف:كتاب غير روائيين كويتيون",
    "Category:Latvian non-fiction writers": "تصنيف:كتاب غير روائيين لاتفيون",
    "Category:Lebanese non-fiction writers": "تصنيف:كتاب غير روائيين لبنانيون",
    "Category:Liberian non-fiction writers": "تصنيف:كتاب غير روائيين ليبيريون",
    "Category:Libyan non-fiction writers": "تصنيف:كتاب غير روائيين ليبيون",
    "Category:Lithuanian non-fiction writers": "تصنيف:كتاب غير روائيين ليتوانيون",
    "Category:Luxembourgian non-fiction writers": "تصنيف:كتاب غير روائيين لوكسمبورغيون",
    "Category:Macedonian non-fiction writers": "تصنيف:كتاب غير روائيين مقدونيون",
    "Category:Malawian non-fiction writers": "تصنيف:كتاب غير روائيين ملاويون",
    "Category:Malaysian non-fiction writers": "تصنيف:كتاب غير روائيين ماليزيون",
    "Category:Male non-fiction writers": "تصنيف:كتاب غير روائيين ذكور",
    "Category:Male non-fiction writers by nationality": "تصنيف:كتاب غير روائيين ذكور حسب الجنسية",
    "Category:Male non-fiction writers from Northern Ireland": "تصنيف:كتاب غير روائيين ذكور من أيرلندا الشمالية",
    "Category:Malian non-fiction writers": "تصنيف:كتاب غير روائيين ماليون",
    "Category:Maltese non-fiction writers": "تصنيف:كتاب غير روائيين مالطيون",
    "Category:Mauritian non-fiction writers": "تصنيف:كتاب غير روائيين موريشيوسيون",
    "Category:Mexican non-fiction writers": "تصنيف:كتاب غير روائيين مكسيكيون",
    "Category:Moroccan non-fiction writers": "تصنيف:كتاب غير روائيين مغاربة",
    "Category:Mozambican non-fiction writers": "تصنيف:كتاب غير روائيين موزمبيقيون",
    "Category:Namibian non-fiction writers": "تصنيف:كتاب غير روائيين ناميبيون",
    "Category:Nigerian non-fiction writers": "تصنيف:كتاب غير روائيين نيجيريون",
    "Category:Non-fiction crime writers": "تصنيف:كتاب جريمة غير روائيين",
    "Category:Non-fiction environmental writers": "تصنيف:كتاب بيئة غير روائيين",
    "Category:Non-fiction writers": "تصنيف:كتاب غير روائيين",
    "Category:Non-fiction writers about organized crime in Italy": "تصنيف:كتاب غير روائيين عن الجريمة المنظمة في إيطاليا",
    "Category:Non-fiction writers about organized crime in the United States": "تصنيف:كتاب غير روائيين عن الجريمة المنظمة في الولايات المتحدة",
    "Category:Non-fiction writers about the French Connection": "تصنيف:كتاب غير روائيين عن الرابط الفرنسي",
    "Category:Non-fiction writers by century": "تصنيف:كتاب غير روائيين حسب القرن",
    "Category:Non-fiction writers by century and nationality": "تصنيف:كتاب غير روائيين حسب القرن والجنسية",
    "Category:Non-fiction writers by nationality": "تصنيف:كتاب غير روائيين حسب الجنسية",
    "Category:Non-fiction writers by nationality and century": "تصنيف:كتاب غير روائيين حسب الجنسية والقرن",
    "Category:Non-fiction writers from Cornwall": "تصنيف:كتاب غير روائيين من كورنوال",
    "Category:Non-fiction writers from Georgia (country)": "تصنيف:كتاب غير روائيين من جورجيا",
    "Category:Non-fiction writers from Northern Ireland": "تصنيف:كتاب غير روائيين من أيرلندا الشمالية",
    "Category:Non-fiction writers from the Ottoman Empire": "تصنيف:كتاب غير روائيين من الدولة العثمانية",
    "Category:Non-fiction writers from the Russian Empire": "تصنيف:كتاب غير روائيين من الإمبراطورية الروسية",
    "Category:Norwegian non-fiction writers": "تصنيف:كتاب غير روائيين نرويجيون",
    "Category:Norwegian non-fiction writers by century": "تصنيف:كتاب غير روائيين نرويجيون حسب القرن",
    "Category:Pakistani non-fiction writers": "تصنيف:كتاب غير روائيين باكستانيون",
    "Category:Pakistani non-fiction writers by century": "تصنيف:كتاب غير روائيين باكستانيون حسب القرن",
    "Category:Palestinian non-fiction writers": "تصنيف:كتاب غير روائيين فلسطينيون",
    "Category:Paraguayan non-fiction writers": "تصنيف:كتاب غير روائيين باراغوايانيون",
    "Category:Peruvian non-fiction writers": "تصنيف:كتاب غير روائيين بيرويون",
    "Category:Polish male non-fiction writers": "تصنيف:كتاب غير روائيين ذكور بولنديون",
    "Category:Polish non-fiction writers": "تصنيف:كتاب غير روائيين بولنديون",
    "Category:Portuguese non-fiction writers": "تصنيف:كتاب غير روائيين برتغاليون",
    "Category:Puerto Rican non-fiction writers": "تصنيف:كتاب غير روائيين بورتوريكيون",
    "Category:Qatari non-fiction writers": "تصنيف:كتاب غير روائيين قطريون",
    "Category:Romanian non-fiction writers": "تصنيف:كتاب غير روائيين رومان",
    "Category:Russian non-fiction writers": "تصنيف:كتاب غير روائيين روس",
    "Category:Russian non-fiction writers by century": "تصنيف:كتاب غير روائيين روس حسب القرن",
    "Category:Rwandan non-fiction writers": "تصنيف:كتاب غير روائيين روانديون",
    "Category:Sahrawi non-fiction writers": "تصنيف:كتاب غير روائيين صحراويون",
    "Category:Saudi Arabian non-fiction writers": "تصنيف:كتاب غير روائيين سعوديون",
    "Category:Scottish non-fiction writers": "تصنيف:كتاب غير روائيين إسكتلنديون",
    "Category:Scottish non-fiction writers by century": "تصنيف:كتاب غير روائيين إسكتلنديون حسب القرن",
    "Category:Senegalese non-fiction writers": "تصنيف:كتاب غير روائيين سنغاليون",
    "Category:Serbian non-fiction writers": "تصنيف:كتاب غير روائيين صرب",
    "Category:Sierra Leonean non-fiction writers": "تصنيف:كتاب غير روائيين سيراليونيون",
    "Category:Singaporean non-fiction writers": "تصنيف:كتاب غير روائيين سنغافوريون",
    "Category:Slovak non-fiction writers": "تصنيف:كتاب غير روائيين سلوفاكيون",
    "Category:Slovenian non-fiction writers": "تصنيف:كتاب غير روائيين سلوفينيون",
    "Category:Somalian non-fiction writers": "تصنيف:كتاب غير روائيين صوماليون",
    "Category:South Korean non-fiction writers": "تصنيف:كتاب غير روائيين كوريون جنوبيون",
    "Category:Soviet non-fiction writers": "تصنيف:كتاب غير روائيين سوفيت",
    "Category:Spanish male non-fiction writers": "تصنيف:كتاب غير روائيين ذكور إسبان",
    "Category:Spanish non-fiction writers": "تصنيف:كتاب غير روائيين إسبان",
    "Category:Spanish non-fiction writers by century": "تصنيف:كتاب غير روائيين إسبان حسب القرن",
    "Category:Sri Lankan non-fiction writers": "تصنيف:كتاب غير روائيين سريلانكيون",
    "Category:Sudanese non-fiction writers": "تصنيف:كتاب غير روائيين سودانيون",
    "Category:Swedish male non-fiction writers": "تصنيف:كتاب غير روائيين ذكور سويديون",
    "Category:Swedish non-fiction writers": "تصنيف:كتاب غير روائيين سويديون",
    "Category:Swedish non-fiction writers by century": "تصنيف:كتاب غير روائيين سويديون حسب القرن",
    "Category:Swiss non-fiction writers": "تصنيف:كتاب غير روائيين سويسريون",
    "Category:Swiss non-fiction writers by century": "تصنيف:كتاب غير روائيين سويسريون حسب القرن",
    "Category:Syrian non-fiction writers": "تصنيف:كتاب غير روائيين سوريون",
    "Category:Taiwanese non-fiction writers": "تصنيف:كتاب غير روائيين تايوانيون",
    "Category:Tajikistani non-fiction writers": "تصنيف:كتاب غير روائيين طاجيك",
    "Category:Tanzanian non-fiction writers": "تصنيف:كتاب غير روائيين تنزانيون",
    "Category:Thai non-fiction writers": "تصنيف:كتاب غير روائيين تايلنديون",
    "Category:Togolese non-fiction writers": "تصنيف:كتاب توغويون غير روائيين",
    "Category:Trinidad and Tobago non-fiction writers": "تصنيف:كتاب غير روائيين ترنيداديون",
    "Category:Tunisian non-fiction writers": "تصنيف:كتاب غير روائيين تونسيون",
    "Category:Turkish Cypriot non-fiction writers": "تصنيف:كتاب غير روائيين قبرصيون شماليون",
    "Category:Turkish non-fiction writers": "تصنيف:كتاب غير روائيين أتراك",
    "Category:Ugandan non-fiction writers": "تصنيف:كتاب غير روائيين أوغنديون",
    "Category:Ukrainian non-fiction writers": "تصنيف:كتاب غير روائيين أوكرانيون",
    "Category:Uruguayan non-fiction writers": "تصنيف:كتاب غير روائيين أوروغويانيون",
    "Category:Venezuelan non-fiction writers": "تصنيف:كتاب غير روائيين فنزويليون",
    "Category:Vietnamese non-fiction writers": "تصنيف:كتاب غير روائيين فيتناميون",
    "Category:Yemeni non-fiction writers": "تصنيف:كتاب غير روائيين يمنيون",
    "Category:Yugoslav non-fiction writers": "تصنيف:كتاب غير روائيين يوغسلافيون",
    "Category:Zambian non-fiction writers": "تصنيف:كتاب غير روائيين زامبيون",
}
to_test = [
    ("text_non_fiction", data_1),
]


@pytest.mark.parametrize("name,data", to_test)
@pytest.mark.skip2
@pytest.mark.dump
def test_all_dump(name: str, data: dict[str, str]) -> None:
    expected, diff_result = one_dump_test(data, resolve_arabic_category_label)

    diff_result2 = {x: v for x, v in diff_result.items() if v}
    # dump_diff(diff_result2, name)

    expected2 = {x: v for x, v in expected.items() if v and x in diff_result2}
    # dump_diff(expected2, f"{name}_expected")

    save3 = [f"* [[:{v}]]>[[:{diff_result2[x]}]]" for x, v in expected.items() if v and x in diff_result2]
    dump_diff(save3, f"{name}_d", _sort=False)

    assert diff_result == expected, f"Differences found: {len(diff_result):,}, len all :{len(data):,}"



# -*- coding: utf-8 -*-


def apply_pattern_replacements(template_label, sport_label, xoxo) -> str:
    """Replace placeholder tokens in a template with the provided sport label."""
    team_lab = ""
    final_label = template_label.replace(xoxo, sport_label)
    if xoxo not in final_label:
        team_lab = final_label
    return team_lab



"""
Tests
"""

import re

import pytest
from load_one_data import dump_diff, one_dump_test

from ArWikiCats.genders_resolvers.nat_genders_pattern_multi import resolve_nat_genders_pattern_v2

test_data_ar = {
    "yemeni softball players": "لاعبو ولاعبات كرة لينة يمنيون",
    "yemeni men's softball players": "لاعبو كرة لينة يمنيون",
    "yemeni male softball players": "لاعبو كرة لينة يمنيون",
    "yemeni women's softball players": "لاعبات كرة لينة يمنيات",
    "Serbian men's footballers": "لاعبو كرة قدم صرب",
    "Russian men's futsal players": "لاعبو كرة صالات روس",
    "Scottish male badminton players": "لاعبو تنس ريشة إسكتلنديون",
    "irish actors": "ممثلون وممثلات أيرلنديون",
    "irish male actors": "ممثلون أيرلنديون",
    "irish actresses": "ممثلات أيرلنديات",
    "Albanian women's volleyball players": "لاعبات كرة طائرة ألبانيات",
    "yemeni women's footballers": "لاعبات كرة قدم يمنيات",
    "yemeni female footballers": "لاعبات كرة قدم يمنيات",
    "American women baseball players": "لاعبات كرة قاعدة أمريكيات",
    "Andorran female tennis players": "لاعبات كرة مضرب أندوريات",
    "Welsh players of Australian rules football": "لاعبو ولاعبات كرة قدم أسترالية ويلزيون",
    "Irish female players of Australian rules football": "لاعبات كرة قدم أسترالية أيرلنديات",
    "Welsh players of american-football": "لاعبو ولاعبات كرة قدم أمريكية ويلزيون",
    "Irish female players of american-football": "لاعبات كرة قدم أمريكية أيرلنديات",
    "Irish female players of american football": "لاعبات كرة قدم أمريكية أيرلنديات",
}

test_data_2 = {}


@pytest.mark.parametrize("category, expected", test_data_ar.items(), ids=test_data_ar.keys())
@pytest.mark.fast
def test_nat_genders_pattern_1(category: str, expected: str) -> None:
    label = resolve_nat_genders_pattern_v2(category)
    assert label == expected


@pytest.mark.parametrize("category, expected", test_data_2.items(), ids=test_data_2.keys())
@pytest.mark.fast
def test_nat_genders_pattern_2(category: str, expected: str) -> None:
    label = resolve_nat_genders_pattern_v2(category)
    assert label == expected


to_test = [
    ("test_nat_genders_pattern_1", test_data_ar, resolve_nat_genders_pattern_v2),
    ("test_nat_genders_pattern_2", test_data_2, resolve_nat_genders_pattern_v2),
]


@pytest.mark.parametrize("name,data,callback", to_test)
@pytest.mark.dump
def test_dump_all(name: str, data: dict[str, str], callback) -> None:
    expected, diff_result = one_dump_test(data, callback)
    dump_diff(diff_result, name)
    assert diff_result == expected, f"Differences found: {len(diff_result):,}, len all :{len(data):,}"



"""

"""

import pytest
from load_one_data import dump_diff, one_dump_test

from ArWikiCats.new_resolvers.jobs_resolvers.relegin_jobs_new import new_religions_jobs_with_suffix

test_religions_data = {
    "painters shi'a muslims": "رسامون مسلمون شيعة",
    "painters shia muslims": "رسامون مسلمون شيعة",
    "painters male muslims": "رسامون ذكور مسلمون",
    "muslims painters": "رسامون مسلمون",
    "painters muslims": "رسامون مسلمون",
}

test_religions_female_data = {
    "female painters shi'a muslims": "رسامات مسلمات شيعيات",
    "painters female shia muslims": "رسامات مسلمات شيعيات",
    "painters women's muslims": "رسامات مسلمات",
    "painters female muslims": "رسامات مسلمات",
    "women's painters muslims": "رسامات مسلمات",
}


@pytest.mark.parametrize("category,expected", test_religions_data.items(), ids=test_religions_data.keys())
@pytest.mark.fast
def test_religions_jobs_1(category: str, expected: str) -> None:
    result = new_religions_jobs_with_suffix(category)
    assert result == expected


@pytest.mark.parametrize("category,expected", test_religions_female_data.items(), ids=test_religions_female_data.keys())
@pytest.mark.fast
def test_religions_females(category: str, expected: str) -> None:
    """Test all nat translation patterns."""
    result = new_religions_jobs_with_suffix(category)
    assert result == expected


TEMPORAL_CASES = [
    ("test_religions_jobs_1", test_religions_data, new_religions_jobs_with_suffix),
    ("test_religions_females", test_religions_female_data, new_religions_jobs_with_suffix),
]


@pytest.mark.parametrize("name,data,callback", TEMPORAL_CASES)
@pytest.mark.dump
def test_all_dump(name: str, data: dict[str, str], callback) -> None:
    expected, diff_result = one_dump_test(data, callback)
    dump_diff(diff_result, name)
    assert diff_result == expected, f"Differences found: {len(diff_result):,}, len all :{len(data):,}"



import pytest

from ArWikiCats.new_resolvers.sports_resolvers.nationalities_and_sports import _load_bot

data = {
    "100 metres in the african championships in athletics": "100 metres in {en} championships in {en_sport}",
    "1330 in south american football": "1330 in {en} {en_sport}",
    "1789 in south american football": "1789 in {en} {en_sport}",
    "1789 in south american women's football": "1789 in {en} women's {en_sport}",
    "1880 european competition for women's football": "1880 {en} competition for women's {en_sport}",
    "1880 european men's handball championship": "1880 {en} men's {en_sport} championship",
    "1880 european women's handball championship": "1880 {en} women's {en_sport} championship",
    "1880 south american women's football championship": "1880 {en} women's {en_sport} championship",
    "the african championships in athletics": "{en} championships in {en_sport}",
    "wheelchair basketball in 2020 parapan american games": "{en_sport} in 2020 parapan {en} games",
    "wheelchair basketball in the asian para games": "{en_sport} in {en} para games",
    "wheelchair basketball in the parapan american games": "{en_sport} in the parapan {en} games",
    "wheelchair basketball playerss in 2020 parapan american games": "{en_sport} playerss in 2020 parapan {en} games",
    "Yemeni football championships": "{en} {en_sport} championships",
    "Yemeni national football teams": "{en} national {en_sport} teams",
}

both_bot_v2 = _load_bot()


@pytest.mark.parametrize("key,expected", data.items(), ids=data.keys())
@pytest.mark.fast
def test_normalize_both(key: str, expected: str) -> None:
    template_label2 = both_bot_v2.normalize_both(key)
    assert template_label2 == expected


data2 = {
    "Yemeni national football teams": "{en} national football teams",
    "1970 european women's handball championship": "1970 {en} women's handball championship",
    "1970 south american women's football championship": "1970 {en} women's football championship",
    "american basketball players by ethnic or national origin": "{en} basketball players by ethnic or national origin",
}


@pytest.mark.parametrize("key,expected", data2.items(), ids=data2.keys())
@pytest.mark.fast
def test_normalize_nat_label(key: str, expected: str) -> None:
    template_label = both_bot_v2.normalize_nat_label(key)
    assert template_label != ""
    assert template_label == expected


data3 = {
    "Yemeni national football teams": "Yemeni national {en_sport} teams",
    "1970 european women's handball championship": "1970 european women's {en_sport} championship",
    "1970 south american women's football championship": "1970 south american women's {en_sport} championship",
    "american basketball players by ethnic or national origin": "american {en_sport} players by ethnic or national origin",
}


@pytest.mark.parametrize("key,expected", data3.items(), ids=data3.keys())
@pytest.mark.fast
def test_normalize_sport_label(key: str, expected: str) -> None:
    template_label = both_bot_v2.normalize_other_label(key)
    assert template_label != ""
    assert template_label == expected



#!/usr/bin/python3
"""
Module for dual-element category translation formatting.

This module provides factory functions for creating formatters that handle
categories with two dynamic elements (e.g., nationality and sport, or
country and profession). It's the primary module for complex category
translations that require pattern matching on multiple elements.

Functions:
    format_multi_data: Creates MultiDataFormatterBase for dual-element translations.
    format_multi_data_v2: Creates MultiDataFormatterBaseV2 with dictionary support.
    get_other_data: Helper to extract templates containing only the second placeholder.

Constants:
    YEAR_PARAM: Default placeholder for the second element ("xoxo").
    COUNTRY_PARAM: Default placeholder for the first element ("natar").

Example:
    >>> from ArWikiCats.translations_formats import format_multi_data
    >>> formatted_data = {"{nat} {sport} players": "لاعبو {sport_ar} {nat_ar}"}
    >>> data_list = {"british": "بريطانيون", "american": "أمريكيون"}
    >>> data_list2 = {"football": "كرة القدم", "basketball": "كرة السلة"}
    >>> bot = format_multi_data(
    ...     formatted_data=formatted_data,
    ...     data_list=data_list,
    ...     data_list2=data_list2,
    ...     key_placeholder="{nat}",
    ...     value_placeholder="{nat_ar}",
    ...     key2_placeholder="{sport}",
    ...     value2_placeholder="{sport_ar}",
    ... )
    >>> bot.search("british football players")
    'لاعبو كرة القدم بريطانيون'

test at tests.translations_formats.test_format_2_data.py
"""

from typing import Dict

from .DataModel import FormatData, FormatDataV2, MultiDataFormatterBase, MultiDataFormatterBaseV2
from .formats_logger import logger

YEAR_PARAM = "xoxo"
COUNTRY_PARAM = "natar"


def get_other_data(
    formatted_data: dict[str, str],
    key_placeholder: str,
    value_placeholder: str,
    key2_placeholder: str,
    value2_placeholder: str,
) -> dict:
    """
    Extract templates that contain only the second placeholder.

    This helper function filters formatted_data to find templates that
    contain the second placeholder (key2_placeholder/value2_placeholder)
    but not the first placeholder (key_placeholder/value_placeholder).
    This is useful for creating a separate formatter for single-element
    translations.

    Args:
        formatted_data: The full template dictionary to filter.
        key_placeholder: First element's key placeholder (to exclude).
        value_placeholder: First element's value placeholder (to exclude).
        key2_placeholder: Second element's key placeholder (to include).
        value2_placeholder: Second element's value placeholder (to include).

    Returns:
        dict: Filtered templates containing only the second placeholder.

    Example:
        >>> formatted_data = {
        ...     "{nat} {sport} players": "لاعبو {sport_ar} {nat_ar}",
        ...     "{sport} coaches": "مدربو {sport_ar}",
        ... }
        >>> other_data = get_other_data(
        ...     formatted_data, "{nat}", "{nat_ar}", "{sport}", "{sport_ar}"
        ... )
        >>> other_data
        {'{sport} coaches': 'مدربو {sport_ar}'}
    """
    other_formatted_data = {
        x: v
        for x, v in formatted_data.items()
        if key2_placeholder in x and key_placeholder not in x and value2_placeholder in v and value_placeholder not in v
    }
    logger.debug(f"len other_formatted_data: {len(other_formatted_data):,}")

    return other_formatted_data


def format_multi_data(
    formatted_data: Dict[str, str],
    data_list: Dict[str, str],
    key_placeholder: str = COUNTRY_PARAM,
    value_placeholder: str = COUNTRY_PARAM,
    data_list2: Dict[str, str] = {},
    key2_placeholder: str = YEAR_PARAM,
    value2_placeholder: str = YEAR_PARAM,
    text_after: str = "",
    text_before: str = "",
    other_formatted_data: Dict[str, str] = {},
    use_other_formatted_data: bool = False,
    search_first_part: bool = False,
    data_to_find: Dict[str, str] | None = None,
    regex_filter: str | None = None,
) -> MultiDataFormatterBase:
    """
    Create a MultiDataFormatterBase for dual-element category translations.

    This factory function creates a formatter that handles categories with
    two dynamic elements (e.g., nationality and sport). It creates two
    internal FormatData instances (country_bot and other_bot) and combines
    them using MultiDataFormatterBase.

    Args:
        formatted_data: Template patterns mapping English patterns to Arabic templates.
            Keys should contain both placeholders (e.g., "{nat} {sport} players").
        data_list: First element key-to-Arabic-label mappings
            (e.g., {"british": "بريطانيون"}).
        key_placeholder: Placeholder for first element key. Default: "natar".
        value_placeholder: Placeholder for first element value. Default: "natar".
        data_list2: Second element key-to-Arabic-label mappings
            (e.g., {"football": "كرة القدم"}).
        key2_placeholder: Placeholder for second element key. Default: "xoxo".
        value2_placeholder: Placeholder for second element value. Default: "xoxo".
        text_after: Optional text that appears after the first element key.
        text_before: Optional text that appears before the first element key.
        use_other_formatted_data: If True, extract single-element templates for other_bot.
        search_first_part: If True, search using only the first part after normalization.
        data_to_find: Optional direct lookup dictionary for category labels.
        regex_filter: Custom regex pattern for word boundary detection.

    Returns:
        MultiDataFormatterBase: A configured formatter for dual-element translations.

    Example:
        >>> formatted_data = {"{nat} {sport} players": "لاعبو {sport_ar} {nat_ar}"}
        >>> data_list = {"british": "بريطانيون"}
        >>> data_list2 = {"football": "كرة القدم"}
        >>> bot = format_multi_data(
        ...     formatted_data=formatted_data,
        ...     data_list=data_list,
        ...     data_list2=data_list2,
        ...     key_placeholder="{nat}",
        ...     value_placeholder="{nat_ar}",
        ...     key2_placeholder="{sport}",
        ...     value2_placeholder="{sport_ar}",
        ... )
        >>> bot.search("british football players")
        'لاعبو كرة القدم بريطانيون'
    """
    # Country bot (FormatData)
    country_bot = FormatData(
        formatted_data=formatted_data,
        data_list=data_list,
        key_placeholder=key_placeholder,
        value_placeholder=value_placeholder,
        text_after=text_after,
        text_before=text_before,
        regex_filter=regex_filter,
    )

    _other_formatted_data = other_formatted_data or (
        get_other_data(
            formatted_data=formatted_data,
            key_placeholder=key_placeholder,
            value_placeholder=value_placeholder,
            key2_placeholder=key2_placeholder,
            value2_placeholder=value2_placeholder,
        )
        if use_other_formatted_data
        else {}
    )

    other_bot = FormatData(
        formatted_data=_other_formatted_data,  # to use from search_all
        data_list=data_list2,
        key_placeholder=key2_placeholder,
        value_placeholder=value2_placeholder,
        regex_filter=regex_filter,
    )

    return MultiDataFormatterBase(
        country_bot=country_bot,
        other_bot=other_bot,
        search_first_part=search_first_part,
        data_to_find=data_to_find,
    )


def format_multi_data_v2(
    formatted_data: Dict[str, str],
    data_list: Dict[str, str],
    key_placeholder: str,
    data_list2: Dict[str, str] = {},
    key2_placeholder: str = YEAR_PARAM,
    text_after: str = "",
    text_before: str = "",
    use_other_formatted_data: bool = False,
    search_first_part: bool = False,
    data_to_find: Dict[str, str] | None = None,
    regex_filter: str | None = None,
) -> MultiDataFormatterBaseV2:
    """
    Create a MultiDataFormatterBaseV2 for dual-element translations with dictionary support.

    This factory function creates a formatter similar to format_multi_data but uses
    FormatDataV2 which supports dictionary values in data_list for complex
    placeholder replacements with multiple values per key.

    Args:
        formatted_data: Template patterns mapping English patterns to Arabic templates.
            Keys should contain both placeholders.
        data_list: First element key-to-value mappings. Values can be strings or
            dictionaries with multiple placeholders.
        key_placeholder: Placeholder for first element key (required).
        data_list2: Second element key-to-value mappings.
        key2_placeholder: Placeholder for second element key. Default: "xoxo".
        text_after: Optional text that appears after the first element key.
        text_before: Optional text that appears before the first element key.
        use_other_formatted_data: If True, extract single-element templates for other_bot.
        search_first_part: If True, search using only the first part after normalization.
        data_to_find: Optional direct lookup dictionary for category labels.
        regex_filter: Custom regex pattern for word boundary detection.

    Returns:
        MultiDataFormatterBaseV2: A configured formatter for dual-element translations.

    Example:
        >>> formatted_data = {"{country} {sport} players": "{demonym} لاعبو {sport_ar}"}
        >>> data_list = {"yemen": {"demonym": "يمنيون"}}
        >>> data_list2 = {"football": {"sport_ar": "كرة القدم"}}
        >>> bot = format_multi_data_v2(
        ...     formatted_data=formatted_data,
        ...     data_list=data_list,
        ...     key_placeholder="{country}",
        ...     data_list2=data_list2,
        ...     key2_placeholder="{sport}",
        ... )
        >>> bot.search("yemen football players")
        'يمنيون لاعبو كرة القدم'
    """
    country_bot = FormatDataV2(
        formatted_data=formatted_data,
        data_list=data_list,
        key_placeholder=key_placeholder,
        text_after=text_after,
        text_before=text_before,
        regex_filter=regex_filter,
    )

    other_formatted_data = (
        {x: v for x, v in formatted_data.items() if key2_placeholder in x and key_placeholder not in x}
        if use_other_formatted_data
        else {}
    )

    other_bot = FormatDataV2(
        formatted_data=other_formatted_data,
        data_list=data_list2,
        key_placeholder=key2_placeholder,
        text_after=text_after,
        text_before=text_before,
        regex_filter=regex_filter,
    )

    return MultiDataFormatterBaseV2(
        country_bot=country_bot,
        other_bot=other_bot,
        search_first_part=search_first_part,
        data_to_find=data_to_find,
    )


__all__ = [
    "format_multi_data",
    "format_multi_data_v2",
]



#!/usr/bin/python3
"""
!
"""

import re

from ..helps import logger
from .pf_keys import change_key_mappings_replacements

# Precompiled Regex Patterns
REGEX_SUB_WHITESPACE = re.compile(r"[\s\t]+", re.IGNORECASE)
REGEX_SUB_CENTURY = re.compile(r"[−–\-]century", re.IGNORECASE)
REGEX_SUB_MILLENNIUM = re.compile(r"[−–\-]millennium", re.IGNORECASE)
REGEX_SUB_MILLENNIUM_CENTURY = re.compile(r"[−–\-](millennium|century)", re.I)
REGEX_SUB_ROYAL_DEFENCE_FORCE = re.compile(r"royal (.*?) defence force", re.IGNORECASE)
REGEX_SUB_ROYAL_NAVAL_FORCE = re.compile(r"royal (.*?) naval force", re.IGNORECASE)
REGEX_SUB_ROYAL_NAVY = re.compile(r"royal (.*?) navy", re.IGNORECASE)
REGEX_SUB_ROYAL_AIR_FORCE = re.compile(r"royal (.*?) air force", re.IGNORECASE)
REGEX_SUB_EXPATRIATE_PEOPLE = re.compile(r"(\w+) expatriate (\w+) people in ", re.IGNORECASE)
REGEX_SUB_ORGANISATIONS = re.compile(r"organisations", re.IGNORECASE)
REGEX_SUB_RUS = re.compile(r"rus'", re.IGNORECASE)
REGEX_SUB_THE_KINGDOM_OF = re.compile(r"the kingdom of", re.IGNORECASE)
REGEX_SUB_AUSTRIA_HUNGARY = re.compile(r"austria-hungary", re.IGNORECASE)
REGEX_SUB_AUSTRIA_HUNGARY_2 = re.compile(r"austria hungary", re.IGNORECASE)
REGEX_SUB_UNMANNED_MILITARY_AIRCRAFT = re.compile(r"unmanned military aircraft of", re.IGNORECASE)
REGEX_SUB_UNMANNED_AERIAL_VEHICLES = re.compile(r"unmanned aerial vehicles of", re.IGNORECASE)
REGEX_SUB_DEMOCRATIC_REPUBLIC_CONGO = re.compile(r"democratic republic of congo", re.IGNORECASE)
REGEX_SUB_REPUBLIC_CONGO = re.compile(r"republic of congo", re.IGNORECASE)
REGEX_SUB_ATHLETICS = re.compile(r"athletics \(track and field\)", re.IGNORECASE)
REGEX_SUB_TWIN_PEOPLE = re.compile(r"twin people", re.IGNORECASE)
REGEX_SUB_PERCENT27 = re.compile(r"\%27", re.IGNORECASE)
REGEX_SUB_CATEGORY_MINISTERS = re.compile(r"category\:ministers of ", re.IGNORECASE)
REGEX_SUB_ASSOCIATION_FOOTBALL_AFC = re.compile(r"association football afc", re.IGNORECASE)
REGEX_SUB_ASSOCIATION_FOOTBALL = re.compile(r"association football", re.IGNORECASE)

replaces = {
    "election, ": "election ",
    "national women's youth": "national youth women's",
    "national youth women's": "national youth women's",
    "women's youth national": "national youth women's",
    "women's national youth": "national youth women's",
    "youth national women's": "national youth women's",
    "youth women's national": "national youth women's",
    "national women's junior": "national junior women's",
    "national junior women's": "national junior women's",
    "women's junior national": "national junior women's",
    "women's national junior": "national junior women's",
    "junior women's national": "national junior women's",
    "national men's junior": "national junior men's",
    "national junior men's": "national junior men's",
    "men's junior national": "national junior men's",
    "men's national junior": "national junior men's",
    "junior men's national": "national junior men's",
    " men's national": " national men's",
    "women's national": "national women's",
    "junior national": "national junior",
    "youth national": "national youth",
    "amateur national": "national amateur",
    "heads of mission ": "heads-of-mission ",
    "house of commons of canada": "house-of-commons-of-canada",
}


def change_cat(cat_orginal: str) -> str:
    """
    Transform and normalize category names by applying various regex patterns and replacements.

    Args:
        cat_orginal: Original category string to transform

    Returns:
        Transformed category string
    """
    cat_orginal = cat_orginal.lower().strip()
    category = cat_orginal

    category = re.sub(r"\bthe\b", "", category, flags=re.IGNORECASE).strip()

    # Normalize whitespace
    category = REGEX_SUB_WHITESPACE.sub(" ", category)

    # Normalize century and millennium formatting
    category = REGEX_SUB_CENTURY.sub(" century", category)
    category = REGEX_SUB_MILLENNIUM.sub(" millennium", category)
    category = REGEX_SUB_MILLENNIUM_CENTURY.sub(r" \g<1>", category)

    # Reorder royal military force names
    category = REGEX_SUB_ROYAL_DEFENCE_FORCE.sub(r"\g<1> royal defence force", category)
    category = REGEX_SUB_ROYAL_NAVAL_FORCE.sub(r"\g<1> royal naval force", category)
    category = REGEX_SUB_ROYAL_NAVY.sub(r"\g<1> royal navy", category)
    category = REGEX_SUB_ROYAL_AIR_FORCE.sub(r"\g<1> royal air force", category)

    # Apply various normalization patterns
    category = REGEX_SUB_EXPATRIATE_PEOPLE.sub(r"\g<1> expatriate \g<2> peoplee in ", category)
    category = REGEX_SUB_ORGANISATIONS.sub("organizations", category)
    category = REGEX_SUB_RUS.sub("rus", category)
    category = REGEX_SUB_THE_KINGDOM_OF.sub(" kingdom of", category)
    category = REGEX_SUB_AUSTRIA_HUNGARY.sub("austria hungary", category)
    category = REGEX_SUB_AUSTRIA_HUNGARY_2.sub("austria hungary", category)
    category = REGEX_SUB_UNMANNED_MILITARY_AIRCRAFT.sub("unmanned military aircraft-of", category)
    category = REGEX_SUB_UNMANNED_AERIAL_VEHICLES.sub("unmanned aerial vehicles-of", category)
    category = REGEX_SUB_DEMOCRATIC_REPUBLIC_CONGO.sub("democratic-republic-of-congo", category)
    category = REGEX_SUB_REPUBLIC_CONGO.sub("republic-of-congo", category)
    category = REGEX_SUB_ATHLETICS.sub("track-and-field athletics", category)
    category = REGEX_SUB_TWIN_PEOPLE.sub("twinpeople", category)
    category = REGEX_SUB_PERCENT27.sub("'", category)

    # Apply simple string replacements
    simple_replacements = {
        "secretaries of ": "secretaries-of ",
        "sportspeople": "sports-people",
        "roller hockey (quad)": "roller hockey",
        "victoria (australia)": "victoria-australia",
        "party of ": "party-of ",
        " uu-16 ": " u-16 ",
    }
    for old, new in simple_replacements.items():
        category = category.replace(old, new)

    # Apply replaces dictionary
    for x, d in replaces.items():
        category = category.replace(x, d)

    category = change_key_mappings_replacements(category)

    # Final transformations
    category = REGEX_SUB_CATEGORY_MINISTERS.sub("category:ministers-of ", category)
    category = REGEX_SUB_ASSOCIATION_FOOTBALL_AFC.sub("association-football afc", category)
    category = REGEX_SUB_ASSOCIATION_FOOTBALL.sub("football", category)

    # Log changes if any
    if category != cat_orginal:
        logger.info(f'change_cat to :"{category}", orginal: {cat_orginal}.')

    return category


__all__ = [
    "change_cat",
]



import os
import sys
from dataclasses import dataclass

argv_lower = [x.lower() for x in sys.argv]


all_params = ["NOKOOORA", "NOWIKIDATA"]


def one_req(name: str) -> bool:
    """Check if the given flag is active via env or command line."""
    all_params.append(name)
    return os.getenv(name.upper(), "false").lower() in ("1", "true", "yes") or name.lower() in argv_lower


@dataclass(frozen=True)
class PrintConfig:
    noprint_formats: bool
    noprint: bool


@dataclass(frozen=True)
class AppConfig:
    start_tgc_resolver_first: bool
    find_stubs: bool
    makeerr: bool
    save_data_path: str


@dataclass(frozen=True)
class Config:
    print: PrintConfig
    app: AppConfig


settings = Config(
    print=PrintConfig(
        noprint=one_req("NOPRINT"),
        noprint_formats=one_req("NOPRINT_FORMATS"),
    ),
    app=AppConfig(
        start_tgc_resolver_first=one_req("TGC_RESOLVER_FIRST"),
        find_stubs=one_req("-STUBS"),
        makeerr=one_req("MAKEERR"),
        save_data_path=os.getenv("SAVE_DATA_PATH", ""),
    ),
)
print_settings = settings.print
app_settings = settings.app

__all__ = [
    "settings",
    "print_settings",
    "app_settings",
    "all_params",
]



import pytest

from ArWikiCats.translations.nats.Nationality import (
    build_american_forms,
    build_lookup_tables,
    load_sources,
    normalize_aliases,
)


def test_load_sources_return_type() -> None:
    data = load_sources()
    assert isinstance(data, dict)
    for v in data.values():
        assert set(v.keys()) == {"male", "males", "female", "females", "en", "ar", "the_male", "the_female"}
        assert all(isinstance(x, str) for x in v.values())


def test_sources_are_merged(monkeypatch) -> None:
    monkeypatch.setattr(
        "ArWikiCats.translations.nats.Nationality.open_json_file", lambda name: {"x": {"en": "test", "ar": "اختبار"}}
    )
    data = load_sources()
    assert "x" in data


def test_alias_mapping() -> None:
    ArWikiCats = {"russian": {"male": "a", "males": "", "female": "", "females": "", "en": "russia", "ar": "روسيا"}}
    ArWikiCats["russians"] = {}  # before normalization
    out = normalize_aliases(ArWikiCats)
    assert out["russians"]["en"] == "russia"


def test_georgia_country_copy() -> None:
    ArWikiCats = {"georgian": {"male": "x", "males": "", "female": "", "females": "", "en": "georgia", "ar": "جورجي"}}
    out = normalize_aliases(ArWikiCats)
    assert out["georgia (country)"]["en"] == "georgia (country)"
    assert out["georgia (country)"]["male"] == "x"


def test_american_form_created() -> None:
    ArWikiCats = {"yemeni": {"male": "يمني", "males": "", "female": "", "females": "", "en": "yemen", "ar": "يمني"}}
    out = build_american_forms(ArWikiCats)
    assert "yemeni-american" in out


def test_no_american_if_no_gender() -> None:
    ArWikiCats = {"abc": {"male": "", "males": "", "female": "", "females": "", "en": "abc", "ar": "abc"}}
    out = build_american_forms(ArWikiCats)
    assert out == {}


def test_jewish_american() -> None:
    ArWikiCats = {"jewish": {"male": "يهودي", "males": "", "female": "", "females": "", "en": "jews", "ar": "يهود"}}
    out = build_american_forms(ArWikiCats)
    assert "jewish-american" in out
    assert "jewish american" in out  # special rule


def test_lookup_nat_men() -> None:
    nat = {"yemeni": {"male": "يمني", "males": "", "female": "", "females": "", "en": "yemen", "ar": "اليمن"}}
    out = build_lookup_tables(nat)
    assert out["Nat_men"]["yemeni"] == "يمني"


def test_country_mapping() -> None:
    nat = {"yemeni": {"male": "يمني", "males": "", "female": "", "females": "", "en": "yemen", "ar": "اليمن"}}
    out = build_lookup_tables(nat)
    assert out["countries_from_nat"]["yemen"] == "اليمن"


def test_the_country_normalization() -> None:
    nat = {
        "british": {
            "male": "بريطاني",
            "males": "",
            "female": "",
            "females": "",
            "en": "the uk",
            "ar": "المملكة المتحدة",
        }
    }
    out = build_lookup_tables(nat)
    assert out["countries_from_nat"]["uk"] == "المملكة المتحدة"


def test_full_pipeline() -> None:
    raw = {"yemeni": {"male": "يمني", "males": "", "female": "يمنية", "females": "", "en": "yemen", "ar": "اليمن"}}

    all_nat = build_american_forms(raw)
    out = build_lookup_tables(all_nat)

    assert "yemeni-american" in all_nat
    assert out["countries_from_nat"].get("yemeni-american") is None
    assert out["Nat_men"]["yemeni-american"] == "أمريكي يمني"


def test_empty_values_handled() -> None:
    raw = {"abc": {"male": "", "males": "", "female": "", "females": "", "en": "", "ar": ""}}
    all_nat2 = build_american_forms(raw)
    assert all_nat2 == {}


def test_uppercase_english_normalized() -> None:
    raw = {"Italian": {"male": "إيطالي", "males": "", "female": "", "females": "", "en": "ITALY", "ar": "إيطاليا"}}
    out = build_lookup_tables(raw)
    assert "italy" in out["countries_from_nat"]
